@@ -40,7 +40,7 @@ PGAPI_BindParameter(HSTMT hstmt,
 					SQLSMALLINT ibScale,
 					PTR rgbValue,
 					SQLLEN cbValueMax,
-					SQLLEN FAR * pcbValue)
+					SQLLEN * pcbValue)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
 	CSTR func = "PGAPI_BindParameter";
@@ -82,7 +82,6 @@ PGAPI_BindParameter(HSTMT hstmt,
 	ipdopts->parameters[ipar].decimal_digits = ibScale;
 	ipdopts->parameters[ipar].precision = 0;
 	ipdopts->parameters[ipar].scale = 0;
-#if (ODBCVER >= 0x0300)
 	switch (fCType)
 	{
 		case SQL_C_NUMERIC:
@@ -104,7 +103,6 @@ PGAPI_BindParameter(HSTMT hstmt,
 	}
 	apdopts->parameters[ipar].precision = ipdopts->parameters[ipar].precision;
 	apdopts->parameters[ipar].scale = ipdopts->parameters[ipar].scale;
-#endif /* ODBCVER */
 
 	/*
 	 * If rebinding a parameter that had data-at-exec stuff in it, then
@@ -134,7 +132,7 @@ PGAPI_BindParameter(HSTMT hstmt,
 #endif /* NOT_USED */
 
 	/* Clear premature result */
-	if (stmt->status == STMT_PREMATURE)
+	if (stmt->status == STMT_DESCRIBED)
 		SC_recycle_statement(stmt);
 
 	mylog("%s: ipar=%d, paramType=%d, fCType=%d, fSqlType=%d, cbColDef=%d, ibScale=%d,", func, ipar, fParamType, fCType, fSqlType, cbColDef, ibScale);
@@ -151,7 +149,7 @@ PGAPI_BindCol(HSTMT hstmt,
 			  SQLSMALLINT fCType,
 			  PTR rgbValue,
 			  SQLLEN cbValueMax,
-			  SQLLEN FAR * pcbValue)
+			  SQLLEN * pcbValue)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
 	CSTR func = "PGAPI_BindCol";
@@ -199,9 +197,7 @@ PGAPI_BindCol(HSTMT hstmt,
 			switch (fCType)
 			{
 				case SQL_C_BOOKMARK:
-#if (ODBCVER >= 0x0300)
 				case SQL_C_VARBOOKMARK:
-#endif /* ODBCVER */
 					break;
 				default:
 					SC_set_error(stmt, STMT_PROGRAM_TYPE_OUT_OF_RANGE, "Bind column 0 is not of type SQL_C_BOOKMARK", func);
@@ -270,7 +266,6 @@ inolog("Bind column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
 		opts->bindings[icol].indicator = pcbValue;
 		opts->bindings[icol].returntype = fCType;
 		opts->bindings[icol].precision = 0;
-#if (ODBCVER >= 0x0300)
 		switch (fCType)
 		{
 			case SQL_C_NUMERIC:
@@ -284,7 +279,6 @@ inolog("Bind column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
 				opts->bindings[icol].precision = 6;
 				break;
 		}
-#endif /* ODBCVER */
 		opts->bindings[icol].scale = 0;
 
 		mylog("       bound buffer[%d] = %p\n", icol, opts->bindings[icol].buffer);
@@ -309,10 +303,10 @@ cleanup:
 RETCODE		SQL_API
 PGAPI_DescribeParam(HSTMT hstmt,
 					SQLUSMALLINT ipar,
-					SQLSMALLINT FAR * pfSqlType,
-					SQLULEN FAR * pcbParamDef,
-					SQLSMALLINT FAR * pibScale,
-					SQLSMALLINT FAR * pfNullable)
+					SQLSMALLINT * pfSqlType,
+					SQLULEN * pcbParamDef,
+					SQLSMALLINT * pibScale,
+					SQLSMALLINT * pfNullable)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
 	CSTR func = "PGAPI_DescribeParam";
@@ -359,7 +353,7 @@ inolog("howTo=%d\n", SC_get_prepare_method(stmt));
 			case NAMED_PARSE_REQUEST:
 			case PARSE_TO_EXEC_ONCE:
 			case PARSE_REQ_FOR_INFO:
-				if (ret = prepareParameters(stmt), SQL_ERROR == ret)
+				if (ret = prepareParameters(stmt, FALSE), SQL_ERROR == ret)
 					goto cleanup;
 		}
 	}
@@ -414,29 +408,6 @@ cleanup:
 }
 
 
-#if (ODBCVER < 0x0300)
-/*	Sets multiple values (arrays) for the set of parameter markers. */
-RETCODE		SQL_API
-PGAPI_ParamOptions(HSTMT hstmt,
-				   SQLULEN crow,
-				   SQLULEN FAR * pirow)
-{
-	CSTR func = "PGAPI_ParamOptions";
-	StatementClass *stmt = (StatementClass *) hstmt;
-	APDFields	*apdopts;
-	IPDFields	*ipdopts;
-
-	mylog("%s: entering... %d %p\n", func, crow, pirow);
-
-	apdopts = SC_get_APDF(stmt);
-	apdopts->paramset_size = crow;
-	ipdopts = SC_get_IPDF(stmt);
-	ipdopts->param_processed_ptr = pirow;
-	return SQL_SUCCESS;
-}
-#endif /* ODBCVER */
-
-
 /*
  *	This function should really talk to the dbms to determine the number of
  *	"parameter markers" (not bound parameters) in the statement.  But, since
@@ -448,7 +419,7 @@ PGAPI_ParamOptions(HSTMT hstmt,
  */
 RETCODE		SQL_API
 PGAPI_NumParams(HSTMT hstmt,
-				SQLSMALLINT FAR * pcpar)
+				SQLSMALLINT * pcpar)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
 	CSTR func = "PGAPI_NumParams";
@@ -486,8 +457,6 @@ inolog("num_params=%d,%d\n", stmt->num_params, stmt->proc_return);
 		stmt->num_params = *pcpar;
 		stmt->proc_return = proc_return;
 		stmt->multi_statement = multi;
-		if (multi)
-			SC_no_parse_tricky(stmt);
 	}
 inolog("num_params=%d,%d\n", stmt->num_params, stmt->proc_return);
 	return SQL_SUCCESS;
