@@ -16,6 +16,8 @@
 #include <string.h>
 #include "descriptor.h"
 
+#include <libpq-fe.h>
+
 #if defined (POSIX_MULTITHREAD_SUPPORT)
 #include <pthread.h>
 #endif
@@ -202,79 +204,6 @@ do { \
 	} \
 } while (0)
 
-/* Authentication types */
-#define AUTH_REQ_OK		0
-#define AUTH_REQ_KRB4		1
-#define AUTH_REQ_KRB5		2
-#define AUTH_REQ_PASSWORD	3
-#define AUTH_REQ_CRYPT		4
-#define AUTH_REQ_MD5		5
-#define AUTH_REQ_SCM_CREDS	6
-#define AUTH_REQ_GSS		7
-#define AUTH_REQ_GSS_CONT	8
-#define AUTH_REQ_SSPI		9
-
-/*	Startup Packet sizes */
-#define SM_DATABASE		64
-#define SM_USER			32
-#define SM_OPTIONS		64
-#define SM_UNUSED		64
-#define SM_TTY			64
-
-/*	Old 6.2 protocol defines */
-#define NO_AUTHENTICATION	7
-#define PATH_SIZE		64
-#define ARGV_SIZE		64
-#define USRNAMEDATALEN		16
-
-typedef unsigned int ProtocolVersion;
-
-#define PG_PROTOCOL(major, minor)	(((major) << 16) | (minor))
-#define PG_PROTOCOL_LATEST	PG_PROTOCOL(3, 0)
-#define PG_PROTOCOL_74	PG_PROTOCOL(3, 0)
-#define PG_PROTOCOL_64	PG_PROTOCOL(2, 0)
-#define PG_PROTOCOL_63	PG_PROTOCOL(1, 0)
-#define PG_PROTOCOL_62	PG_PROTOCOL(0, 0)
-#define PG_NEGOTIATE_SSLMODE	PG_PROTOCOL(1234, 5679)
-
-/*	This startup packet is to support latest Postgres protocol (6.4, 6.3) */
-typedef struct _StartupPacket
-{
-	ProtocolVersion protoVersion;
-	char		database[SM_DATABASE];
-	char		user[SM_USER];
-	char		options[SM_OPTIONS];
-	char		unused[SM_UNUSED];
-	char		tty[SM_TTY];
-} StartupPacket;
-
-
-/*	This startup packet is to support pre-Postgres 6.3 protocol */
-typedef struct _StartupPacket6_2
-{
-	unsigned int authtype;
-	char		database[PATH_SIZE];
-	char		user[USRNAMEDATALEN];
-	char		options[ARGV_SIZE];
-	char		execfile[ARGV_SIZE];
-	char		tty[PATH_SIZE];
-} StartupPacket6_2;
-
-/* Transferred from pqcomm.h:  */
-
-
-typedef ProtocolVersion MsgType;
-
-#define CANCEL_REQUEST_CODE PG_PROTOCOL(1234,5678)
-
-typedef struct CancelRequestPacket
-{
-	/* Note that each field is stored in network byte order! */
-	MsgType		cancelRequestCode;	/* code to identify a cancel request */
-	unsigned int	backendPID;	/* PID of client's backend */
-	unsigned int	cancelAuthCode; /* secret key to authorize cancel */
-} CancelRequestPacket;
-
 /*	Structure to hold all the connection attributes for a specific
 	connection (used for both registry and file, DSN and DRIVER)
 */
@@ -287,7 +216,6 @@ typedef struct
 	char		database[MEDIUM_REGISTRY_LEN];
 	char		username[MEDIUM_REGISTRY_LEN];
 	pgNAME		password;
-	char		protocol[SMALL_REGISTRY_LEN];
 	char		port[SMALL_REGISTRY_LEN];
 	char		sslmode[16];
 	char		onlyread[SMALL_REGISTRY_LEN];
@@ -297,12 +225,8 @@ typedef struct
 	char		show_system_tables[SMALL_REGISTRY_LEN];
 	char		translation_dll[MEDIUM_REGISTRY_LEN];
 	char		translation_option[SMALL_REGISTRY_LEN];
-	char		focus_password;
-#ifdef	USE_LIBPQ
-	signed char	prefer_libpq;
-#endif /* USE_LIBPQ */
+	char		password_required;
 	pgNAME		conn_settings;
-	signed char	disallow_premature;
 	signed char	allow_keyset;
 	signed char	updatable_cursors;
 	signed char	lf_conversion;
@@ -316,7 +240,6 @@ typedef struct
 	signed char	bde_environment;
 	signed char	fake_mss;
 	signed char	cvt_null_date_string;
-	signed char	autocommit_public;
 	signed char	accessible_only;
 	signed char	ignore_round_trip_time;
 	signed char	disable_keepalive;
@@ -330,22 +253,8 @@ typedef struct
 	GLOBAL_VALUES drivers;		/* moved from driver's option */
 } ConnInfo;
 
-/*	Macro to determine is the connection using 6.2 protocol? */
-#define PROTOCOL_62(conninfo_)		(strncmp((conninfo_)->protocol, PG62, strlen(PG62)) == 0)
+#define SUPPORT_DESCRIBE_PARAM(conninfo_) (conninfo_->use_server_side_prepare)
 
-/*	Macro to determine is the connection using 6.3 protocol? */
-#define PROTOCOL_63(conninfo_)		(strncmp((conninfo_)->protocol, PG63, strlen(PG63)) == 0)
-
-/*	Macro to determine is the connection using 6.4 protocol? */
-#define PROTOCOL_64(conninfo_)		(strncmp((conninfo_)->protocol, PG64, strlen(PG64)) == 0)
-
-/*	Macro to determine is the connection using 7.4 protocol? */
-#define PROTOCOL_74(conninfo_)		(strncmp((conninfo_)->protocol, PG74, strlen(PG74)) == 0)
-
-/*	Macro to determine is the connection using 7.4 rejected? */
-#define PROTOCOL_74REJECTED(conninfo_)	(strncmp((conninfo_)->protocol, PG74REJECTED, strlen(PG74REJECTED)) == 0)
-
-#define SUPPORT_DESCRIBE_PARAM(conninfo_) (PROTOCOL_74(conninfo_) && conninfo_->use_server_side_prepare)
 /*
  *	Macros to compare the server's version with a specified version
  *		1st parameter: pointer to a ConnectionClass object
@@ -360,13 +269,8 @@ typedef struct
 	((conn)->pg_version_major == major && (conn)->pg_version_minor >= minor))
 #define SERVER_VERSION_EQ(conn, major, minor) \
 	((conn)->pg_version_major == major && (conn)->pg_version_minor == minor)
-#define SERVER_VERSION_LE(conn, major, minor) (! SERVER_VERSION_GT(conn, major, minor))
-#define SERVER_VERSION_LT(conn, major, minor) (! SERVER_VERSION_GE(conn, major, minor))
-/*#if ! defined(HAVE_CONFIG_H) || defined(HAVE_STRINGIZE)*/
 #define STRING_AFTER_DOT(string)	(strchr(#string, '.') + 1)
-/*#else
-#define STRING_AFTER_DOT(str)	(strchr("str", '.') + 1)
-#endif*/
+
 /*
  *	Simplified macros to compare the server's version with a
  *		specified version
@@ -385,7 +289,6 @@ typedef struct
 /*	This is used to store cached table information in the connection */
 struct col_info
 {
-	Int2		num_reserved_cols;
 	Int2		refcnt;
 	QResultClass	*result;
 	pgNAME		schema_name;
@@ -415,12 +318,12 @@ struct col_info
 #define HINSTANCE void *
 #endif
 
-typedef BOOL (FAR WINAPI * DataSourceToDriverProc) (UDWORD, SWORD, PTR,
-		SDWORD, PTR, SDWORD, SDWORD FAR *, UCHAR FAR *, SWORD,
-		SWORD FAR *);
-typedef BOOL (FAR WINAPI * DriverToDataSourceProc) (UDWORD, SWORD, PTR,
-		SDWORD, PTR, SDWORD, SDWORD FAR *, UCHAR FAR *, SWORD,
-		SWORD FAR *);
+typedef BOOL (WINAPI * DataSourceToDriverProc) (UDWORD, SWORD, PTR,
+		SDWORD, PTR, SDWORD, SDWORD *, UCHAR *, SWORD,
+		SWORD *);
+typedef BOOL (WINAPI * DriverToDataSourceProc) (UDWORD, SWORD, PTR,
+		SDWORD, PTR, SDWORD, SDWORD *, UCHAR *, SWORD,
+		SWORD *);
 
 /*******	The Connection handle	************/
 struct ConnectionClass_
@@ -428,6 +331,7 @@ struct ConnectionClass_
 	HENV		henv;		/* environment this connection was
 					 * created on */
 	SQLUINTEGER	login_timeout;
+	signed char	autocommit_public;
 	StatementOptions stmtOptions;
 	ARDFields	ardOptions;
 	APDFields	apdOptions;
@@ -439,7 +343,7 @@ struct ConnectionClass_
 	StatementClass	**stmts;
 	Int2		num_stmts;
 	Int2		ncursors;
-	SocketClass	*sock;
+	PGconn	   *pqconn;
 	Int4		lobj_type;
 	Int2		coli_allocated;
 	Int2		ntables;
@@ -448,7 +352,6 @@ struct ConnectionClass_
 	HINSTANCE	translation_handle;
 	DataSourceToDriverProc DataSourceToDriver;
 	DriverToDataSourceProc DriverToDataSource;
-	Int2		driver_version;		/* prepared for ODBC3.0 */
 	char		transact_status;	/* Is a transaction is currently
 						 * in progress */
 	char		errormsg_created;	/* has an informative error msg
@@ -456,38 +359,30 @@ struct ConnectionClass_
 	char		pg_version[MAX_INFO_STRING];	/* Version of PostgreSQL
 							 * we're connected to -
 							 * DJP 25-1-2001 */
-	float		pg_version_number;
 	Int2		pg_version_major;
 	Int2		pg_version_minor;
 	char		ms_jet;
 	char		unicode;
 	char		result_uncommitted;
-	char		schema_support;
 	char		lo_is_domain;
-	char		escape_in_literal;
 	char		*original_client_encoding;
-	char		*current_client_encoding;
 	char		*server_encoding;
 	Int2		ccsc;
 	Int2		mb_maxbyte_per_char;
-	int		be_pid;	/* pid returned by backend */
-	int		be_key; /* auth code needed to send cancel */
 	UInt4		isolation;
 	char		*current_schema;
-	StatementClass	*stmt_in_extquery;
+	char		current_schema_valid;	/* is current_schema valid? TRUE when
+										 * current_schema == NULL means it's
+										 * really NULL, while FALSE means it's
+										 * unknown */
+	StatementClass *unnamed_prepared_stmt;
 	Int2		max_identifier_length;
 	Int2		num_discardp;
 	char		**discardp;
-#if (ODBCVER >= 0x0300)
 	int		num_descs;
 	DescriptorClass	**descs;
-#endif /* ODBCVER */
 	pgNAME		schemaIns;
 	pgNAME		tableIns;
-#ifdef	USE_SSPI
-	UInt4		svcs_allowed;
-	UInt4		auth_svcs;
-#endif /* USE_SSPI */
 #if defined(WIN_MULTITHREAD_SUPPORT)
 	CRITICAL_SECTION	cs;
 	CRITICAL_SECTION	slock;
@@ -504,15 +399,13 @@ struct ConnectionClass_
 
 /* Accessor functions */
 #define CC_get_env(x)				((x)->henv)
-#define CC_get_socket(x)			(x->sock)
 #define CC_get_database(x)			(x->connInfo.database)
 #define CC_get_server(x)			(x->connInfo.server)
 #define CC_get_DSN(x)				(x->connInfo.dsn)
 #define CC_get_username(x)			(x->connInfo.username)
 #define CC_is_onlyread(x)			(x->connInfo.onlyread[0] == '1')
-#define CC_get_escape(x)			(x->escape_in_literal)
 #define CC_fake_mss(x)	(/* 0 != (x)->ms_jet && */ 0 < (x)->connInfo.fake_mss)
-#define CC_accessible_only(x)	(0 < (x)->connInfo.accessible_only && PG_VERSION_GE((x), 7.2))
+#define CC_accessible_only(x)	(0 < (x)->connInfo.accessible_only)
 #define CC_default_is_c(x)	(CC_is_in_ansi_app(x) || x->ms_jet /* not only */ || TRUE /* but for any other ? */)
 /*	for CC_DSN_info */
 #define CONN_DONT_OVERWRITE		0
@@ -558,31 +451,26 @@ char		CC_commit(ConnectionClass *self);
 char		CC_abort(ConnectionClass *self);
 char		CC_set_autocommit(ConnectionClass *self, BOOL on);
 int		CC_set_translation(ConnectionClass *self);
-char		CC_connect(ConnectionClass *self, char password_req, char *salt);
+char		CC_connect(ConnectionClass *self, char *salt);
 char		CC_add_statement(ConnectionClass *self, StatementClass *stmt);
 char		CC_remove_statement(ConnectionClass *self, StatementClass *stmt)
 ;
-#if (ODBCVER >= 0x0300)
 char		CC_add_descriptor(ConnectionClass *self, DescriptorClass *desc);
 char		CC_remove_descriptor(ConnectionClass *self, DescriptorClass *desc);
-#endif /* ODBCVER */
 void		CC_set_error(ConnectionClass *self, int number, const char *message, const char *func);
 void		CC_set_errormsg(ConnectionClass *self, const char *message);
 char		CC_get_error(ConnectionClass *self, int *number, char **message);
 QResultClass *CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UDWORD flag, StatementClass *stmt, const char *appendq);
 #define CC_send_query(self, query, qi, flag, stmt) CC_send_query_append(self, query, qi, flag, stmt, NULL)
+void		handle_pgres_error(ConnectionClass *self, const PGresult *pgres,
+				   const char *comment,
+				   QResultClass *res, BOOL fatal);
 void		CC_clear_error(ConnectionClass *self);
-int		CC_send_function(ConnectionClass *conn, int fnid, void *result_buf, int *actual_result_len, int result_is_int, LO_ARG *argv, int nargs);
+int		CC_send_function(ConnectionClass *conn, const char *fn_name, void *result_buf, int *actual_result_len, int result_is_int, LO_ARG *argv, int nargs);
 char		CC_send_settings(ConnectionClass *self);
-/*
-char		*CC_create_errormsg(ConnectionClass *self);
-void		CC_lookup_lo(ConnectionClass *conn);
-void		CC_lookup_pg_version(ConnectionClass *conn);
-*/
 void		CC_initialize_pg_version(ConnectionClass *conn);
 void		CC_log_error(const char *func, const char *desc, const ConnectionClass *self);
-int		CC_get_max_query_len(const ConnectionClass *self);
-int		CC_send_cancel_request(const ConnectionClass *conn);
+int			CC_send_cancel_request(const ConnectionClass *conn);
 void		CC_on_commit(ConnectionClass *conn);
 void		CC_on_abort(ConnectionClass *conn, UDWORD opt);
 void		CC_on_abort_partial(ConnectionClass *conn);
@@ -591,15 +479,8 @@ const char	*CC_get_current_schema(ConnectionClass *conn);
 int             CC_mark_a_object_to_discard(ConnectionClass *conn, int type, const char *plan);
 int             CC_discard_marked_objects(ConnectionClass *conn);
 
-int	handle_error_message(ConnectionClass *self, char *msgbuf, size_t buflen,
-		 char *sqlstate, const char *comment, QResultClass *res);
-int	handle_notice_message(ConnectionClass *self, char *msgbuf, size_t buflen,
-		 char *sqlstate, const char *comment, QResultClass *res);
-int		EatReadyForQuery(ConnectionClass *self);
-void		getParameterValues(ConnectionClass *self);
 int		CC_get_max_idlen(ConnectionClass *self);
-
-BOOL		SendSyncRequest(ConnectionClass *self);
+char	CC_get_escape(const ConnectionClass *self);
 
 const		char *CurrCat(const ConnectionClass *self);
 const		char *CurrCatString(const ConnectionClass *self);
@@ -609,11 +490,10 @@ void	CC_examine_global_transaction(ConnectionClass *self);
 /* CC_send_query options */
 enum {
 	IGNORE_ABORT_ON_CONN	= 1L /* not set the error result even when  */
-	,CREATE_KEYSET		= (1L << 1) /* create keyset for updatable curosrs */
-	,GO_INTO_TRANSACTION	= (1L << 2) /* issue begin in advance */
+	,CREATE_KEYSET		= (1L << 1) /* create keyset for updatable cursors */
+	,GO_INTO_TRANSACTION	= (1L << 2) /* issue BEGIN in advance */
 	,ROLLBACK_ON_ERROR	= (1L << 3) /* rollback the query when an error occurs */
-	,END_WITH_COMMIT	= (1L << 4) /* the query ends with COMMMIT command */
-	,IGNORE_ROUND_TRIP	= (1L << 5) /* the commincation round trip time is considered ignorable */
+	,END_WITH_COMMIT	= (1L << 4) /* the query ends with COMMIT command */
 };
 /* CC_on_abort options */
 #define	NO_TRANS		1L
