@@ -28,16 +28,6 @@ void	TI_Constructor(TABLE_INFO *self, const ConnectionClass *conn)
 {
 	memset(self, 0, sizeof(TABLE_INFO));
 	TI_set_updatable(self);
-	if (PG_VERSION_LT(conn, 7.2))
-	{
-		char	qual[32];
-
-		STR_TO_NAME(self->bestitem, OID_NAME);
-		sprintf(qual, "\"%s\" = %%u", OID_NAME);
-		STRX_TO_NAME(self->bestqual, qual);
-		TI_set_hasoids(self);
-		TI_set_hasoids_checked(self);
-	}
 }
 void	TI_Destructor(TABLE_INFO **ti, int count)
 {
@@ -228,9 +218,7 @@ void
 InitializeARDFields(ARDFields *opt)
 {
 	memset(opt, 0, sizeof(ARDFields));
-#if (ODBCVER >= 0x0300)
 	opt->size_of_rowset = 1;
-#endif /* ODBCVER */
 	opt->bind_size = 0;		/* default is to bind by column */
 	opt->size_of_rowset_odbc2 = 1;
 }
@@ -256,8 +244,6 @@ BindInfoClass	*ARD_AllocBookmark(ARDFields *ardopts)
 	return ardopts->bookmark;
 }
 
-#if (ODBCVER >= 0x0300)
-
 #define	DESC_INCREMENT	10
 char CC_add_descriptor(ConnectionClass *self, DescriptorClass *desc)
 {
@@ -565,11 +551,11 @@ int	DC_get_errornumber(const DescriptorClass *desc)
 }
 
 /*	Map sql commands to statement types */
-static struct
+static const struct
 {
-	int	number;
-	const	char	* ver3str;
-	const	char	* ver2str;
+	int		number;
+	const char	ver3str[6];
+	const char	ver2str[6];
 }	Descriptor_sqlstate[] =
 
 {
@@ -651,11 +637,11 @@ DC_log_error(const char *func, const char *desc, const DescriptorClass *self)
 RETCODE		SQL_API
 PGAPI_DescError(SQLHDESC hdesc,
 				SQLSMALLINT	RecNumber,
-				SQLCHAR FAR * szSqlState,
-				SQLINTEGER FAR * pfNativeError,
-				SQLCHAR FAR * szErrorMsg,
+				SQLCHAR * szSqlState,
+				SQLINTEGER * pfNativeError,
+				SQLCHAR * szErrorMsg,
 				SQLSMALLINT cbErrorMsgMax,
-				SQLSMALLINT FAR * pcbErrorMsg,
+				SQLSMALLINT * pcbErrorMsg,
 				UWORD flag)
 {
 	CSTR func = "PGAPI_DescError";
@@ -669,5 +655,3 @@ PGAPI_DescError(SQLHDESC hdesc,
 				pfNativeError, szErrorMsg, cbErrorMsgMax,
 				pcbErrorMsg, flag);
 }
-
-#endif /* ODBCVER */
