@@ -98,9 +98,7 @@ typedef struct DescriptorHeader_
  */
 struct ARDFields_
 {
-#if (ODBCVER >= 0x0300)
 	SQLLEN		size_of_rowset; /* for ODBC3 fetch operation */
-#endif /* ODBCVER */
 	SQLUINTEGER	bind_size;	/* size of each structure if using
 					 * Row-wise Binding */
 	SQLUSMALLINT	*row_operation_ptr;
@@ -194,7 +192,6 @@ void	ARD_unbind_cols(ARDFields *self, BOOL freeall);
 void	APD_free_params(APDFields *self, char option);
 void	IPD_free_params(IPDFields *self, char option);
 BOOL	getCOLIfromTI(const char *, ConnectionClass *, StatementClass *, const OID, TABLE_INFO **);
-#if (ODBCVER >= 0x0300)
 RETCODE	DC_set_stmt(DescriptorClass *desc, StatementClass *stmt);
 void	DC_clear_error(DescriptorClass *desc);
 void	DC_set_error(DescriptorClass *desc, int errornumber, const char * errormsg);
@@ -203,7 +200,6 @@ PG_ErrorInfo *DC_get_error(DescriptorClass *self);
 int	DC_get_errornumber(const DescriptorClass *self);
 const char *DC_get_errormsg(const DescriptorClass *self);
 void	DC_log_error(const char *func, const char *desc, const DescriptorClass *self);
-#endif /* ODBCVER */
 
 /*	Error numbers about descriptor handle */
 enum {
