@@ -220,24 +220,6 @@ makeKeepaliveConnectString(char *target, const ConnInfo *ci, BOOL abbrev)
 	return target;
 }
 
-#ifdef	USE_LIBPQ
-static char *
-makePreferLibpqConnectString(char *target, const ConnInfo *ci, BOOL abbrev)
-{
-	char	*buf = target;
-	*buf = '\0';
-
-	if (ci->prefer_libpq < 0)
-		return target;
-
-	if (abbrev)
-		sprintf(buf, ABBR_PREFERLIBPQ "=%u;", ci->prefer_libpq);
-	else
-		sprintf(buf, INI_PREFERLIBPQ "=%u;", ci->prefer_libpq);
-	return target;
-}
-#endif /* USE_LIBPQ */
-
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 char *
 makeXaOptConnectString(char *target, const ConnInfo *ci, BOOL abbrev)
@@ -264,10 +246,7 @@ makeConnectString(char *connect_string, const ConnInfo *ci, UWORD len)
 {
 	char		got_dsn = (ci->dsn[0] != '\0');
 	char		encoded_item[LARGE_REGISTRY_LEN];
-	char		keepaliveStr[32];
-#ifdef	USE_LIBPQ
-	char		preferLibpqStr[32];
-#endif
+	char		keepaliveStr[64];
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	char		xaOptStr[16];
 #endif
@@ -305,9 +284,9 @@ inolog("hlen=%d", hlen);
 		char	protocol_and[16];
 
 		if (ci->rollback_on_error >= 0)
-			snprintf(protocol_and, sizeof(protocol_and), "%s-%d", ci->protocol, ci->rollback_on_error);
+			snprintf(protocol_and, sizeof(protocol_and), "7.4-%d", ci->rollback_on_error);
 		else
-			strncpy_null(protocol_and, ci->protocol, sizeof(protocol_and));
+			strcpy(protocol_and, "7.4");
 		olen = snprintf(&connect_string[hlen], nlen, ";"
 			INI_SSLMODE "=%s;"
 			INI_READONLY "=%s;"
@@ -318,33 +297,25 @@ inolog("hlen=%d", hlen);
 			INI_SHOWSYSTEMTABLES "=%s;"
 			INI_CONNSETTINGS "=%s;"
 			INI_FETCH "=%d;"
-			INI_SOCKET "=%d;"
 			INI_UNKNOWNSIZES "=%d;"
 			INI_MAXVARCHARSIZE "=%d;"
 			INI_MAXLONGVARCHARSIZE "=%d;"
 			INI_DEBUG "=%d;"
 			INI_COMMLOG "=%d;"
-			INI_OPTIMIZER "=%d;"
-			INI_KSQO "=%d;"
 			INI_USEDECLAREFETCH "=%d;"
 			INI_TEXTASLONGVARCHAR "=%d;"
 			INI_UNKNOWNSASLONGVARCHAR "=%d;"
 			INI_BOOLSASCHAR "=%d;"
 			INI_PARSE "=%d;"
-			INI_CANCELASFREESTMT "=%d;"
 			INI_EXTRASYSTABLEPREFIXES "=%s;"
 			INI_LFCONVERSION "=%d;"
 			INI_UPDATABLECURSORS "=%d;"
-			INI_DISALLOWPREMATURE "=%d;"
 			INI_TRUEISMINUS1 "=%d;"
 			INI_INT8AS "=%d;"
 			INI_BYTEAASLONGVARBINARY "=%d;"
 			INI_USESERVERSIDEPREPARE "=%d;"
 			INI_LOWERCASEIDENTIFIER "=%d;"
 			"%s"
-#ifdef	USE_LIBPQ
-			"%s"
-#endif /* USE_LIBPQ */
 #ifdef	WIN32
 			INI_GSSAUTHUSEGSSAPI "=%d;"
 #endif /* WIN32 */
@@ -360,33 +331,25 @@ inolog("hlen=%d", hlen);
 			,ci->show_system_tables
 			,encoded_item
 			,ci->drivers.fetch_max
-			,ci->drivers.socket_buffersize
 			,ci->drivers.unknown_sizes
 			,ci->drivers.max_varchar_size
 			,ci->drivers.max_longvarchar_size
 			,ci->drivers.debug
 			,ci->drivers.commlog
-			,ci->drivers.disable_optimizer
-			,ci->drivers.ksqo
 			,ci->drivers.use_declarefetch
 			,ci->drivers.text_as_longvarchar
 			,ci->drivers.unknowns_as_longvarchar
 			,ci->drivers.bools_as_char
 			,ci->drivers.parse
-			,ci->drivers.cancel_as_freestmt
 			,ci->drivers.extra_systable_prefixes
 			,ci->lf_conversion
 			,ci->allow_keyset
-			,ci->disallow_premature
 			,ci->true_is_minus1
 			,ci->int8_as
 			,ci->bytea_as_longvarbinary
 			,ci->use_server_side_prepare
 			,ci->lower_case_identifier
 			,makeKeepaliveConnectString(keepaliveStr, ci, FALSE)
-#ifdef	USE_LIBPQ
-			,makePreferLibpqConnectString(preferLibpqStr, ci, FALSE)
-#endif /* USE_LIBPQ */
 #ifdef	WIN32
 			,ci->gssauth_use_gssapi
 #endif /* WIN32 */
@@ -399,20 +362,12 @@ inolog("hlen=%d", hlen);
 	if (abbrev || olen >= nlen || olen < 0)
 	{
 		flag = 0;
-		if (ci->disallow_premature)
-			flag |= BIT_DISALLOWPREMATURE;
 		if (ci->allow_keyset)
 			flag |= BIT_UPDATABLECURSORS;
 		if (ci->lf_conversion)
 			flag |= BIT_LFCONVERSION;
 		if (ci->drivers.unique_index)
 			flag |= BIT_UNIQUEINDEX;
-		if (PROTOCOL_74(ci))
-			flag |= (BIT_PROTOCOL_64 | BIT_PROTOCOL_63);
-		else if (PROTOCOL_64(ci))
-			flag |= BIT_PROTOCOL_64;
-		else if (PROTOCOL_63(ci))
-			flag |= BIT_PROTOCOL_63;
 		switch (ci->drivers.unknown_sizes)
 		{
 			case UNKNOWNS_AS_DONTKNOW:
@@ -422,18 +377,12 @@ inolog("hlen=%d", hlen);
 				flag |= BIT_UNKNOWN_ASMAX;
 				break;
 		}
-		if (ci->drivers.disable_optimizer)
-			flag |= BIT_OPTIMIZER;
-		if (ci->drivers.ksqo)
-			flag |= BIT_KSQO;
 		if (ci->drivers.commlog)
 			flag |= BIT_COMMLOG;
 		if (ci->drivers.debug)
 			flag |= BIT_DEBUG;
 		if (ci->drivers.parse)
 			flag |= BIT_PARSE;
-		if (ci->drivers.cancel_as_freestmt)
-			flag |= BIT_CANCELASFREESTMT;
 		if (ci->drivers.use_declarefetch)
 			flag |= BIT_USEDECLAREFETCH;
 		if (ci->onlyread[0] == '1')
@@ -467,7 +416,7 @@ inolog("hlen=%d", hlen);
 		{
 			char	abbrevmode[sizeof(ci->sslmode)];
 
-			olen = snprintf(&connect_string[hlen], nlen, ";"
+			(void) snprintf(&connect_string[hlen], nlen, ";"
 				ABBR_SSLMODE "=%s", abbrev_sslmode(ci->sslmode, abbrevmode));
 		}
 		hlen = strlen(connect_string);
@@ -475,35 +424,27 @@ inolog("hlen=%d", hlen);
 		olen = snprintf(&connect_string[hlen], nlen, ";"
 				ABBR_CONNSETTINGS "=%s;"
 				ABBR_FETCH "=%d;"
-				ABBR_SOCKET "=%d;"
 				ABBR_MAXVARCHARSIZE "=%d;"
 				ABBR_MAXLONGVARCHARSIZE "=%d;"
 				INI_INT8AS "=%d;"
 				ABBR_EXTRASYSTABLEPREFIXES "=%s;"
 				"%s"
-#ifdef	USE_LIBPQ
-				"%s"
-#endif /* USE_LIBPQ */
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 				"%s"
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 				INI_ABBREVIATE "=%02x%x",
 				encoded_item,
 				ci->drivers.fetch_max,
-				ci->drivers.socket_buffersize,
 				ci->drivers.max_varchar_size,
 				ci->drivers.max_longvarchar_size,
 				ci->int8_as,
 				ci->drivers.extra_systable_prefixes,
 				makeKeepaliveConnectString(keepaliveStr, ci, TRUE),
-#ifdef	USE_LIBPQ
-				makePreferLibpqConnectString(preferLibpqStr, ci, TRUE),
-#endif /* USE_LIBPQ */
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 				makeXaOptConnectString(xaOptStr, ci, TRUE),
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 				EFFECTIVE_BIT_COUNT, flag);
-		if (olen < nlen && (PROTOCOL_74(ci) || ci->rollback_on_error >= 0))
+		if (olen < nlen || ci->rollback_on_error >= 0)
 		{
 			hlen = strlen(connect_string);
 			nlen = MAX_CONNECT_STRING - hlen;
@@ -513,12 +454,11 @@ inolog("hlen=%d", hlen);
 			 */
 			if (ci->rollback_on_error >= 0)
 				olen = snprintf(&connect_string[hlen], nlen, ";"
-				ABBR_PROTOCOL "=%s-%d",
-				ci->protocol, ci->rollback_on_error);
+				ABBR_PROTOCOL "=7.4-%d",
+								ci->rollback_on_error);
 			else
 				olen = snprintf(&connect_string[hlen], nlen, ";"
-				ABBR_PROTOCOL "=%s",
-				ci->protocol);
+								ABBR_PROTOCOL "=7.4");
 		}
 	}
 	if (olen < nlen)
@@ -556,35 +496,20 @@ unfoldCXAttribute(ConnInfo *ci, const char *value)
 		sscanf(cnt, "%x", &count);
 		sscanf(value + 2, "%x", &flag);
 	}
-	ci->disallow_premature = (char)((flag & BIT_DISALLOWPREMATURE) != 0);
 	ci->allow_keyset = (char)((flag & BIT_UPDATABLECURSORS) != 0);
 	ci->lf_conversion = (char)((flag & BIT_LFCONVERSION) != 0);
 	if (count < 4)
 		return;
 	ci->drivers.unique_index = (char)((flag & BIT_UNIQUEINDEX) != 0);
-	if ((flag & BIT_PROTOCOL_64) != 0)
-	{
-		if ((flag & BIT_PROTOCOL_63) != 0)
-			strcpy(ci->protocol, PG74);
-		else
-			strcpy(ci->protocol, PG64);
-	}
-	else if ((flag & BIT_PROTOCOL_63) != 0)
-		strcpy(ci->protocol, PG63);
-	else
-		strcpy(ci->protocol, PG62);
 	if ((flag & BIT_UNKNOWN_DONTKNOW) != 0)
 		ci->drivers.unknown_sizes = UNKNOWNS_AS_DONTKNOW;
 	else if ((flag & BIT_UNKNOWN_ASMAX) != 0)
 		ci->drivers.unknown_sizes = UNKNOWNS_AS_MAX;
 	else
 		ci->drivers.unknown_sizes = UNKNOWNS_AS_LONGEST;
-	ci->drivers.disable_optimizer = (char)((flag & BIT_OPTIMIZER) != 0);
-	ci->drivers.ksqo = (char)((flag & BIT_KSQO) != 0);
 	ci->drivers.commlog = (char)((flag & BIT_COMMLOG) != 0);
 	ci->drivers.debug = (char)((flag & BIT_DEBUG) != 0);
 	ci->drivers.parse = (char)((flag & BIT_PARSE) != 0);
-	ci->drivers.cancel_as_freestmt = (char)((flag & BIT_CANCELASFREESTMT) != 0);
 	ci->drivers.use_declarefetch = (char)((flag & BIT_USEDECLAREFETCH) != 0);
 	sprintf(ci->onlyread, "%d", (char)((flag & BIT_READONLY) != 0));
 	ci->drivers.text_as_longvarchar = (char)((flag & BIT_TEXTASLONGVARCHAR) !=0);
@@ -636,20 +561,24 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 	else if (stricmp(attribute, INI_PROTOCOL) == 0 || stricmp(attribute, ABBR_PROTOCOL) == 0)
 	{
 		char	*ptr;
-
+		/*
+		 * The first part of the Protocol used to be "6.2", "6.3" or
+		 * "7.4" to denote which protocol version to use. Nowadays we
+		 * only support the 7.4 protocol, also known as the protocol
+		 * version 3. So just ignore the first part of the string,
+		 * parsing only the rollback_on_error value.
+		 */
 		ptr = strchr(value, '-');
 		if (ptr)
 		{
 			if ('-' != *value)
 			{
 				*ptr = '\0';
-				strcpy(ci->protocol, value);
+				/* ignore first part */
 			}
 			ci->rollback_on_error = atoi(ptr + 1);
 			mylog("rollback_on_error=%d\n", ci->rollback_on_error);
 		}
-		else
-			strcpy(ci->protocol, value);
 	}
 
 	else if (stricmp(attribute, INI_SHOWOIDCOLUMN) == 0 || stricmp(attribute, ABBR_SHOWOIDCOLUMN) == 0)
@@ -679,8 +608,6 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		else
 			ci->conn_settings = decode(value);
 	}
-	else if (stricmp(attribute, INI_DISALLOWPREMATURE) == 0 || stricmp(attribute, ABBR_DISALLOWPREMATURE) == 0)
-		ci->disallow_premature = atoi(value);
 	else if (stricmp(attribute, INI_UPDATABLECURSORS) == 0 || stricmp(attribute, ABBR_UPDATABLECURSORS) == 0)
 		ci->allow_keyset = atoi(value);
 	else if (stricmp(attribute, INI_LFCONVERSION) == 0 || stricmp(attribute, ABBR_LFCONVERSION) == 0)
@@ -701,10 +628,6 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		ci->keepalive_idle = atoi(value);
 	else if (stricmp(attribute, INI_KEEPALIVEINTERVAL) == 0 || stricmp(attribute, ABBR_KEEPALIVEINTERVAL) == 0)
 		ci->keepalive_interval = atoi(value);
-#ifdef	USE_LIBPQ
-	else if (stricmp(attribute, INI_PREFERLIBPQ) == 0 || stricmp(attribute, ABBR_PREFERLIBPQ) == 0)
-		ci->prefer_libpq = atoi(value);
-#endif /* USE_LIBPQ */
 	else if (stricmp(attribute, INI_SSLMODE) == 0 || stricmp(attribute, ABBR_SSLMODE) == 0)
 	{
 		switch (value[0])
@@ -766,7 +689,7 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 	else
 		found = FALSE;
 
-	mylog("%s: DSN='%s',server='%s',dbase='%s',user='%s',passwd='%s',port='%s',onlyread='%s',protocol='%s',conn_settings='%s',disallow_premature=%d)\n", func, ci->dsn, ci->server, ci->database, ci->username, NAME_IS_VALID(ci->password) ? "xxxxx" : "", ci->port, ci->onlyread, ci->protocol, ci->conn_settings, ci->disallow_premature);
+	mylog("%s: DSN='%s',server='%s',dbase='%s',user='%s',passwd='%s',port='%s',onlyread='%s',conn_settings='%s')\n", func, ci->dsn, ci->server, ci->database, ci->username, NAME_IS_VALID(ci->password) ? "xxxxx" : "", ci->port, ci->onlyread, ci->conn_settings);
 
 	return found;
 }
@@ -779,17 +702,10 @@ copyCommonAttributes(ConnInfo *ci, const char *attribute, const char *value)
 
 	if (stricmp(attribute, INI_FETCH) == 0 || stricmp(attribute, ABBR_FETCH) == 0)
 		ci->drivers.fetch_max = atoi(value);
-	else if (stricmp(attribute, INI_SOCKET) == 0 || stricmp(attribute, ABBR_SOCKET) == 0)
-		ci->drivers.socket_buffersize = atoi(value);
 	else if (stricmp(attribute, INI_DEBUG) == 0 || stricmp(attribute, ABBR_DEBUG) == 0)
 		ci->drivers.debug = atoi(value);
 	else if (stricmp(attribute, INI_COMMLOG) == 0 || stricmp(attribute, ABBR_COMMLOG) == 0)
 		ci->drivers.commlog = atoi(value);
-	else if (stricmp(attribute, INI_OPTIMIZER) == 0 || stricmp(attribute, ABBR_OPTIMIZER) == 0)
-		ci->drivers.disable_optimizer = atoi(value);
-	else if (stricmp(attribute, INI_KSQO) == 0 || stricmp(attribute, ABBR_KSQO) == 0)
-		ci->drivers.ksqo = atoi(value);
-
 	/*
 	 * else if (stricmp(attribute, INI_UNIQUEINDEX) == 0 ||
 	 * stricmp(attribute, "UIX") == 0) ci->drivers.unique_index =
@@ -801,8 +717,6 @@ copyCommonAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		ci->drivers.lie = atoi(value);
 	else if (stricmp(attribute, INI_PARSE) == 0 || stricmp(attribute, ABBR_PARSE) == 0)
 		ci->drivers.parse = atoi(value);
-	else if (stricmp(attribute, INI_CANCELASFREESTMT) == 0 || stricmp(attribute, ABBR_CANCELASFREESTMT) == 0)
-		ci->drivers.cancel_as_freestmt = atoi(value);
 	else if (stricmp(attribute, INI_USEDECLAREFETCH) == 0 || stricmp(attribute, ABBR_USEDECLAREFETCH) == 0)
 		ci->drivers.use_declarefetch = atoi(value);
 	else if (stricmp(attribute, INI_MAXVARCHARSIZE) == 0 || stricmp(attribute, ABBR_MAXVARCHARSIZE) == 0)
@@ -820,22 +734,18 @@ copyCommonAttributes(ConnInfo *ci, const char *attribute, const char *value)
 	else
 		found = FALSE;
 
-	mylog("%s: A7=%d;A8=%d;A9=%d;B0=%d;B1=%d;B2=%d;B3=%d;B4=%d;B5=%d;B6=%d;B7=%d;B8=%d;B9=%d;C0=%d;C1=%d;C2=%s", func,
+	mylog("%s: A7=%d;A9=%d;B0=%d;B1=%d;B2=%d;B3=%d;B6=%d;B7=%d;B8=%d;B9=%d;C0=%d;C2=%s", func,
 		  ci->drivers.fetch_max,
-		  ci->drivers.socket_buffersize,
 		  ci->drivers.unknown_sizes,
 		  ci->drivers.max_varchar_size,
 		  ci->drivers.max_longvarchar_size,
 		  ci->drivers.debug,
 		  ci->drivers.commlog,
-		  ci->drivers.disable_optimizer,
-		  ci->drivers.ksqo,
 		  ci->drivers.use_declarefetch,
 		  ci->drivers.text_as_longvarchar,
 		  ci->drivers.unknowns_as_longvarchar,
 		  ci->drivers.bools_as_char,
 		  ci->drivers.parse,
-		  ci->drivers.cancel_as_freestmt,
 		  ci->drivers.extra_systable_prefixes);
 
 	return found;
@@ -847,15 +757,9 @@ getDSNdefaults(ConnInfo *ci)
 {
 	mylog("calling getDSNdefaults\n");
 
-	if (ci->port[0] == '\0')
-		strcpy(ci->port, DEFAULT_PORT);
-
 	if (ci->onlyread[0] == '\0')
 		sprintf(ci->onlyread, "%d", globals.onlyread);
 
-	if (ci->protocol[0] == '\0')
-		strcpy(ci->protocol, globals.protocol);
-
 	if (ci->fake_oid_index[0] == '\0')
 		sprintf(ci->fake_oid_index, "%d", DEFAULT_FAKEOIDINDEX);
 
@@ -868,8 +772,6 @@ getDSNdefaults(ConnInfo *ci)
 	if (ci->row_versioning[0] == '\0')
 		sprintf(ci->row_versioning, "%d", DEFAULT_ROWVERSIONING);
 
-	if (ci->disallow_premature < 0)
-		ci->disallow_premature = DEFAULT_DISALLOWPREMATURE;
 	if (ci->allow_keyset < 0)
 		ci->allow_keyset = DEFAULT_UPDATABLECURSORS;
 	if (ci->lf_conversion < 0)
@@ -996,11 +898,12 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 	if (ci->show_system_tables[0] == '\0' || overwrite)
 		SQLGetPrivateProfileString(DSN, INI_SHOWSYSTEMTABLES, "", ci->show_system_tables, sizeof(ci->show_system_tables), ODBC_INI);
 
-	if (ci->protocol[0] == '\0' || overwrite)
+	if (ci->rollback_on_error == -1 || overwrite)
 	{
+		char protocol[SMALL_REGISTRY_LEN];
 		char	*ptr;
-		SQLGetPrivateProfileString(DSN, INI_PROTOCOL, "", ci->protocol, sizeof(ci->protocol), ODBC_INI);
-		if (ptr = strchr(ci->protocol, '-'), NULL != ptr)
+		SQLGetPrivateProfileString(DSN, INI_PROTOCOL, "", protocol, sizeof(protocol), ODBC_INI);
+		if (ptr = strchr(protocol, '-'), NULL != ptr)
 		{
 			*ptr = '\0';
 			if (overwrite || ci->rollback_on_error < 0)
@@ -1023,13 +926,6 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 	if (ci->translation_option[0] == '\0' || overwrite)
 		SQLGetPrivateProfileString(DSN, INI_TRANSLATIONOPTION, "", ci->translation_option, sizeof(ci->translation_option), ODBC_INI);
 
-	if (ci->disallow_premature < 0 || overwrite)
-	{
-		SQLGetPrivateProfileString(DSN, INI_DISALLOWPREMATURE, "", temp, sizeof(temp), ODBC_INI);
-		if (temp[0])
-			ci->disallow_premature = atoi(temp);
-	}
-
 	if (ci->allow_keyset < 0 || overwrite)
 	{
 		SQLGetPrivateProfileString(DSN, INI_UPDATABLECURSORS, "", temp, sizeof(temp), ODBC_INI);
@@ -1100,14 +996,6 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 			if (0 == (ci->keepalive_interval = atoi(temp)))
 				ci->keepalive_interval = -1;
 	}
-#ifdef	USE_LIBPQ
-	if (ci->prefer_libpq < 0 || overwrite)
-	{
-		SQLGetPrivateProfileString(DSN, INI_PREFERLIBPQ, "-1", temp, sizeof(temp), ODBC_INI);
-		if (temp[0])
-			ci->prefer_libpq = atoi(temp);
-	}
-#endif /* USE_LIBPQ */
 
 	if (ci->sslmode[0] == '\0' || overwrite)
 		SQLGetPrivateProfileString(DSN, INI_SSLMODE, "", ci->sslmode, sizeof(ci->sslmode), ODBC_INI);
@@ -1143,9 +1031,8 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 		 ci->database,
 		 ci->username,
 		 NAME_IS_VALID(ci->password) ? "xxxxx" : "");
-	qlog("          onlyread='%s',protocol='%s',showoid='%s',fakeoidindex='%s',showsystable='%s'\n",
+	qlog("          onlyread='%s',showoid='%s',fakeoidindex='%s',showsystable='%s'\n",
 		 ci->onlyread,
-		 ci->protocol,
 		 ci->show_oid_column,
 		 ci->fake_oid_index,
 		 ci->show_system_tables);
@@ -1196,14 +1083,6 @@ writeDriverCommoninfo(const char *fileName, const char *sectionName,
 	if (!SQLWritePrivateProfileString(sectionName, INI_FETCH, tmp, fileName))
 		errc--;
 
-	sprintf(tmp, "%d", comval->disable_optimizer);
-	if (!SQLWritePrivateProfileString(sectionName, INI_OPTIMIZER, tmp, fileName))
-		errc--;
-
-	sprintf(tmp, "%d", comval->ksqo);
-	if (!SQLWritePrivateProfileString(sectionName, INI_KSQO, tmp, fileName))
-		errc--;
-
 	sprintf(tmp, "%d", comval->unique_index);
 	if (!SQLWritePrivateProfileString(sectionName, INI_UNIQUEINDEX, tmp, fileName))
 		errc--;
@@ -1241,10 +1120,6 @@ writeDriverCommoninfo(const char *fileName, const char *sectionName,
 	if (!SQLWritePrivateProfileString(sectionName, INI_PARSE, tmp, fileName))
 		errc--;
 
-	sprintf(tmp, "%d", comval->cancel_as_freestmt);
-	if (!SQLWritePrivateProfileString(sectionName, INI_CANCELASFREESTMT, tmp, fileName))
-		errc--;
-
 	sprintf(tmp, "%d", comval->max_varchar_size);
 	if (!SQLWritePrivateProfileString(sectionName, INI_MAXVARCHARSIZE, tmp, fileName))
 		errc--;
@@ -1332,9 +1207,9 @@ writeDSNinfo(const ConnInfo *ci)
 								 ODBC_INI);
 
 	if (ci->rollback_on_error >= 0)
-		sprintf(temp, "%s-%d", ci->protocol, ci->rollback_on_error);
+		sprintf(temp, "7.4-%d", ci->rollback_on_error);
 	else
-		strncpy_null(temp, ci->protocol, sizeof(temp));
+		strncpy_null(temp, "", sizeof(temp));
 	SQLWritePrivateProfileString(DSN,
 								 INI_PROTOCOL,
 								 temp,
@@ -1346,11 +1221,6 @@ writeDSNinfo(const ConnInfo *ci)
 								 encoded_item,
 								 ODBC_INI);
 
-	sprintf(temp, "%d", ci->disallow_premature);
-	SQLWritePrivateProfileString(DSN,
-								 INI_DISALLOWPREMATURE,
-								 temp,
-								 ODBC_INI);
 	sprintf(temp, "%d", ci->allow_keyset);
 	SQLWritePrivateProfileString(DSN,
 								 INI_UPDATABLECURSORS,
@@ -1410,13 +1280,6 @@ writeDSNinfo(const ConnInfo *ci)
 								 INI_KEEPALIVEINTERVAL,
 								 temp,
 								 ODBC_INI);
-#ifdef	USE_LIBPQ
-	sprintf(temp, "%d", ci->prefer_libpq);
-	SQLWritePrivateProfileString(DSN,
-								 INI_PREFERLIBPQ,
-								 temp,
-								 ODBC_INI);
-#endif /* USE_LIBPQ */
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	sprintf(temp, "%d", ci->xa_opt);
 	SQLWritePrivateProfileString(DSN, INI_XAOPT, temp, ODBC_INI);
@@ -1455,14 +1318,6 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 	else if (inst_position)
 		comval->fetch_max = FETCH_MAX;
 
-	/* Socket Buffersize is stored in driver section */
-	SQLGetPrivateProfileString(section, INI_SOCKET, "",
-							   temp, sizeof(temp), filename);
-	if (temp[0])
-		comval->socket_buffersize = atoi(temp);
-	else if (inst_position)
-		comval->socket_buffersize = SOCK_BUFFER_SIZE;
-
 	/* Debug is stored in the driver section */
 	SQLGetPrivateProfileString(section, INI_DEBUG, "",
 							   temp, sizeof(temp), filename);
@@ -1481,21 +1336,6 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 
 	if (!ci)
 		logs_on_off(0, 0, 0);
-	/* Optimizer is stored in the driver section only */
-	SQLGetPrivateProfileString(section, INI_OPTIMIZER, "",
-							   temp, sizeof(temp), filename);
-	if (temp[0])
-		comval->disable_optimizer = atoi(temp);
-	else if (inst_position)
-		comval->disable_optimizer = DEFAULT_OPTIMIZER;
-
-	/* KSQO is stored in the driver section only */
-	SQLGetPrivateProfileString(section, INI_KSQO, "",
-							   temp, sizeof(temp), filename);
-	if (temp[0])
-		comval->ksqo = atoi(temp);
-	else if (inst_position)
-		comval->ksqo = DEFAULT_KSQO;
 
 	/* Recognize Unique Index is stored in the driver section only */
 	SQLGetPrivateProfileString(section, INI_UNIQUEINDEX, "",
@@ -1505,7 +1345,6 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 	else if (inst_position)
 		comval->unique_index = DEFAULT_UNIQUEINDEX;
 
-
 	/* Unknown Sizes is stored in the driver section only */
 	SQLGetPrivateProfileString(section, INI_UNKNOWNSIZES, "",
 							   temp, sizeof(temp), filename);
@@ -1531,14 +1370,6 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 	else if (inst_position)
 		comval->parse = DEFAULT_PARSE;
 
-	/* SQLCancel calls SQLFreeStmt in Driver Manager */
-	SQLGetPrivateProfileString(section, INI_CANCELASFREESTMT, "",
-							   temp, sizeof(temp), filename);
-	if (temp[0])
-		comval->cancel_as_freestmt = atoi(temp);
-	else if (inst_position)
-		comval->cancel_as_freestmt = DEFAULT_CANCELASFREESTMT;
-
 	/* UseDeclareFetch is stored in the driver section only */
 	SQLGetPrivateProfileString(section, INI_USEDECLAREFETCH, "",
 							   temp, sizeof(temp), filename);
