@@ -34,7 +34,7 @@
 
 extern GLOBAL_VALUES globals;
 
-extern HINSTANCE NEAR s_hModule;
+extern HINSTANCE s_hModule;
 static int	driver_optionsDraw(HWND, const ConnInfo *, int src, BOOL enable);
 static int	driver_options_update(HWND hdlg, ConnInfo *ci, const char *);
 
@@ -60,7 +60,6 @@ void
 SetDlgStuff(HWND hdlg, const ConnInfo *ci)
 {
 	char	buff[MEDIUM_REGISTRY_LEN + 1];
-	BOOL	libpq_exist = FALSE;
 	int	i, dsplevel, selidx, dspcount;
 
 	/*
@@ -77,24 +76,13 @@ SetDlgStuff(HWND hdlg, const ConnInfo *ci)
 	SetDlgItemText(hdlg, IDC_PORT, ci->port);
 
 	dsplevel = 0;
-#ifdef USE_LIBPQ
-	libpq_exist = SSLLIB_check();
-mylog("libpq_exist=%d\n", libpq_exist);
-	if (libpq_exist)
-	{
-		ShowWindow(GetDlgItem(hdlg, IDC_NOTICE_USER), SW_HIDE);
-		dsplevel = 2;
-	}
-	else
-#endif /* USE_LIBPQ */
-	{
-mylog("SendMessage CTL_COLOR\n");
-		SendMessage(GetDlgItem(hdlg, IDC_NOTICE_USER), WM_CTLCOLOR, 0, 0);
-#ifdef	USE_SSPI
-		ShowWindow(GetDlgItem(hdlg, IDC_NOTICE_USER), SW_HIDE);
-		dsplevel = 2;
-#endif /* USE_SSPI */
-	}
+
+	/*
+	 * XXX: We used to hide or show this depending whether libpq was loaded,
+	 * but we depend on libpq directly nowadays, so it's always loaded.
+	 */
+	ShowWindow(GetDlgItem(hdlg, IDC_NOTICE_USER), SW_HIDE);
+	dsplevel = 2;
 
 	selidx = -1;
 	for (i = 0; i < sizeof(modetab) / sizeof(modetab[0]); i++)
@@ -161,14 +149,11 @@ driver_optionsDraw(HWND hdlg, const ConnInfo *ci, int src, BOOL enable)
 			if (!defset)
 			{
 				defval.commlog = DEFAULT_COMMLOG;
-				defval.disable_optimizer = DEFAULT_OPTIMIZER;
-				defval.ksqo = DEFAULT_KSQO;
 				defval.unique_index = DEFAULT_UNIQUEINDEX;
 				defval.onlyread = DEFAULT_READONLY;
 				defval.use_declarefetch = DEFAULT_USEDECLAREFETCH;
 
 				defval.parse = DEFAULT_PARSE;
-				defval.cancel_as_freestmt = DEFAULT_CANCELASFREESTMT;
 				defval.debug = DEFAULT_DEBUG;
 
 				/* Unknown Sizes */
@@ -189,8 +174,6 @@ driver_optionsDraw(HWND hdlg, const ConnInfo *ci, int src, BOOL enable)
 #ifndef Q_LOG
 	EnableWindow(GetDlgItem(hdlg, DRV_COMMLOG), FALSE);
 #endif /* Q_LOG */
-	CheckDlgButton(hdlg, DRV_OPTIMIZER, comval->disable_optimizer);
-	CheckDlgButton(hdlg, DRV_KSQO, comval->ksqo);
 	CheckDlgButton(hdlg, DRV_UNIQUEINDEX, comval->unique_index);
 	/* EnableWindow(GetDlgItem(hdlg, DRV_UNIQUEINDEX), enable); */
 	CheckDlgButton(hdlg, DRV_READONLY, comval->onlyread);
@@ -220,7 +203,6 @@ driver_optionsDraw(HWND hdlg, const ConnInfo *ci, int src, BOOL enable)
 	CheckDlgButton(hdlg, DRV_UNKNOWNS_LONGVARCHAR, comval->unknowns_as_longvarchar);
 	CheckDlgButton(hdlg, DRV_BOOLS_CHAR, comval->bools_as_char);
 	CheckDlgButton(hdlg, DRV_PARSE, comval->parse);
-	CheckDlgButton(hdlg, DRV_CANCELASFREESTMT, comval->cancel_as_freestmt);
 	CheckDlgButton(hdlg, DRV_DEBUG, comval->debug);
 #ifndef MY_LOG
 	EnableWindow(GetDlgItem(hdlg, DRV_DEBUG), FALSE);
@@ -248,8 +230,6 @@ driver_options_update(HWND hdlg, ConnInfo *ci, const char *updateDriver)
 	else
 		comval = &globals;
 	comval->commlog = IsDlgButtonChecked(hdlg, DRV_COMMLOG);
-	comval->disable_optimizer = IsDlgButtonChecked(hdlg, DRV_OPTIMIZER);
-	comval->ksqo = IsDlgButtonChecked(hdlg, DRV_KSQO);
 	comval->unique_index = IsDlgButtonChecked(hdlg, DRV_UNIQUEINDEX);
 	if (!ci)
 	{
@@ -273,7 +253,6 @@ driver_options_update(HWND hdlg, ConnInfo *ci, const char *updateDriver)
 
 	comval->parse = IsDlgButtonChecked(hdlg, DRV_PARSE);
 
-	comval->cancel_as_freestmt = IsDlgButtonChecked(hdlg, DRV_CANCELASFREESTMT);
 	comval->debug = IsDlgButtonChecked(hdlg, DRV_DEBUG);
 
 	comval->fetch_max = GetDlgItemInt(hdlg, DRV_CACHE_SIZE, NULL, FALSE);
@@ -522,17 +501,6 @@ ds_options_update(HWND hdlg, ConnInfo *ci)
 	/* Readonly */
 	sprintf(ci->onlyread, "%d", IsDlgButtonChecked(hdlg, DS_READONLY));
 
-	/* Protocol */
-	if (IsDlgButtonChecked(hdlg, DS_PG62))
-		strcpy(ci->protocol, PG62);
-	else if (IsDlgButtonChecked(hdlg, DS_PG63))
-		strcpy(ci->protocol, PG63);
-	else if (IsDlgButtonChecked(hdlg, DS_PG64))
-		strcpy(ci->protocol, PG64);
-	else
-		/* latest */
-		strcpy(ci->protocol, PG74);
-
 	/* Issue rollback command on error */
 	if (IsDlgButtonChecked(hdlg, DS_NO_ROLLBACK))
 		ci->rollback_on_error = 0;
@@ -563,7 +531,6 @@ ds_options_update(HWND hdlg, ConnInfo *ci)
 	sprintf(ci->show_system_tables, "%d", IsDlgButtonChecked(hdlg, DS_SHOWSYSTEMTABLES));
 
 	sprintf(ci->row_versioning, "%d", IsDlgButtonChecked(hdlg, DS_ROWVERSIONING));
-	ci->disallow_premature = IsDlgButtonChecked(hdlg, DS_DISALLOWPREMATURE);
 	ci->lf_conversion = IsDlgButtonChecked(hdlg, DS_LFCONVERSION);
 	ci->true_is_minus1 = IsDlgButtonChecked(hdlg, DS_TRUEISMINUS1);
 	ci->allow_keyset = IsDlgButtonChecked(hdlg, DS_UPDATABLECURSORS);
@@ -650,19 +617,6 @@ ds_options2Proc(HWND hdlg,
 
 			/* Protocol */
 			enable = (ci->sslmode[0] == SSLLBYTE_DISABLE || ci->username[0] == '\0');
-			EnableWindow(GetDlgItem(hdlg, DS_PG62), enable);
-			EnableWindow(GetDlgItem(hdlg, DS_PG63), enable);
-			EnableWindow(GetDlgItem(hdlg, DS_PG64), enable);
-			EnableWindow(GetDlgItem(hdlg, DS_PG74), enable);
-			if (PROTOCOL_62(ci))
-				CheckDlgButton(hdlg, DS_PG62, 1);
-			else if (PROTOCOL_63(ci))
-				CheckDlgButton(hdlg, DS_PG63, 1);
-			else if (PROTOCOL_64(ci))
-				CheckDlgButton(hdlg, DS_PG64, 1);
-			else
-				/* latest */
-				CheckDlgButton(hdlg, DS_PG74, 1);
 
 			/* How to issue Rollback */
 			switch (ci->rollback_on_error)
@@ -706,7 +660,6 @@ ds_options2Proc(HWND hdlg,
 			CheckDlgButton(hdlg, DS_FAKEOIDINDEX, atoi(ci->fake_oid_index));
 			CheckDlgButton(hdlg, DS_ROWVERSIONING, atoi(ci->row_versioning));
 			CheckDlgButton(hdlg, DS_SHOWSYSTEMTABLES, atoi(ci->show_system_tables));
-			CheckDlgButton(hdlg, DS_DISALLOWPREMATURE, ci->disallow_premature);
 			CheckDlgButton(hdlg, DS_LFCONVERSION, ci->lf_conversion);
 			CheckDlgButton(hdlg, DS_TRUEISMINUS1, ci->true_is_minus1);
 			CheckDlgButton(hdlg, DS_UPDATABLECURSORS, ci->allow_keyset);
@@ -715,9 +668,6 @@ ds_options2Proc(HWND hdlg,
 			/*CheckDlgButton(hdlg, DS_LOWERCASEIDENTIFIER, ci->lower_case_identifier);*/
 			CheckDlgButton(hdlg, DS_GSSAUTHUSEGSSAPI, ci->gssauth_use_gssapi);
 
-#if	!defined(USE_LIBPQ) && !defined(USE_SSPI) && !defined(USE_GSS)
-			EnableWindow(GetDlgItem(hdlg, DS_GSSAUTHUSEGSSAPI), FALSE);
-#endif
 			EnableWindow(GetDlgItem(hdlg, DS_FAKEOIDINDEX), atoi(ci->show_oid_column));
 
 			/* Datasource Connection Settings */
@@ -789,21 +739,6 @@ ds_options3Draw(HWND hdlg, const ConnInfo *ci)
 	BOOL	enable = TRUE;
 	static BOOL defset = FALSE;
 
-	/* The use of LIBPQ library */
-#ifdef	USE_LIBPQ
-	if (ci->prefer_libpq < 0)
-		CheckDlgButton(hdlg, DS_DEFAULT_LIBPQ_USE, 1);
-	else if (0 == ci->prefer_libpq)
-		CheckDlgButton(hdlg, DS_NO_LIBPQ_USE, 1);
-	else 
-		CheckDlgButton(hdlg, DS_LIBPQ_USE, 1);
-#else
-	enable = FALSE;
-	EnableWindow(GetDlgItem(hdlg, DS_DEFAULT_LIBPQ_USE), enable);
-	EnableWindow(GetDlgItem(hdlg, DS_LIBPQ_USE), enable);
-	EnableWindow(GetDlgItem(hdlg, DS_NO_LIBPQ_USE), enable);
-#endif /* USE_LIBPQ */
-
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	switch (ci->xa_opt)
 	{
@@ -838,16 +773,6 @@ ds_options3_update(HWND hdlg, ConnInfo *ci)
 {
 	mylog("%s: got ci = %p\n", __FUNCTION__, ci);
 
-#ifdef	USE_LIBPQ
-	/* Libpq use */
-	if (IsDlgButtonChecked(hdlg, DS_DEFAULT_LIBPQ_USE))
-		ci->prefer_libpq = -1;
-	else if (IsDlgButtonChecked(hdlg, DS_NO_LIBPQ_USE))
-		ci->prefer_libpq = 0;
-	else
-		ci->prefer_libpq = 1;
-#endif /* USE_LIBPQ */
-
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	if (IsDlgButtonChecked(hdlg, DS_DTC_LINK_ONLY))
 		ci->xa_opt = DTC_CHECK_LINK_ONLY;
