@@ -23,7 +23,6 @@
 #ifndef WIN32
 #include <sys/types.h>
 #include <sys/socket.h>
-#define NEAR
 #else
 #include <winsock2.h>
 #endif
@@ -69,18 +68,18 @@ static void dconn_get_common_attributes(const char *connect_string, ConnInfo *ci
 LRESULT CALLBACK dconn_FDriverConnectProc(HWND hdlg, UINT wMsg, WPARAM wParam, LPARAM lParam);
 RETCODE		dconn_DoDialog(HWND hwnd, ConnInfo *ci);
 
-extern HINSTANCE NEAR s_hModule;	/* Saved module handle. */
+extern HINSTANCE s_hModule;	/* Saved module handle. */
 #endif
 
 
 RETCODE		SQL_API
 PGAPI_DriverConnect(HDBC hdbc,
 					HWND hwnd,
-					const SQLCHAR FAR * szConnStrIn,
+					const SQLCHAR * szConnStrIn,
 					SQLSMALLINT cbConnStrIn,
-					SQLCHAR FAR * szConnStrOut,
+					SQLCHAR * szConnStrOut,
 					SQLSMALLINT cbConnStrOutMax,
-					SQLSMALLINT FAR * pcbConnStrOut,
+					SQLSMALLINT * pcbConnStrOut,
 					SQLUSMALLINT fDriverCompletion)
 {
 	CSTR func = "PGAPI_DriverConnect";
@@ -96,7 +95,6 @@ PGAPI_DriverConnect(HDBC hdbc,
 	char		connStrOut[MAX_CONNECT_STRING];
 	int			retval;
 	char		salt[5];
-	char		password_required = AUTH_REQ_OK;
 	ssize_t		len = 0;
 	SQLSMALLINT	lenStrout;
 
@@ -154,8 +152,6 @@ PGAPI_DriverConnect(HDBC hdbc,
 #ifdef WIN32
 dialog:
 #endif
-	ci->focus_password = password_required;
-
 inolog("DriverCompletion=%d\n", fDriverCompletion);
 	switch (fDriverCompletion)
 	{
@@ -173,12 +169,10 @@ inolog("DriverCompletion=%d\n", fDriverCompletion);
 
 		case SQL_DRIVER_COMPLETE:
 
-			paramRequired = password_required;
+			paramRequired = (ci->password_required && NAME_IS_NULL(ci->password));
 			/* Password is not a required parameter. */
 			if (ci->database[0] == '\0')
 				paramRequired = TRUE;
-			else if (ci->port[0] == '\0')
-				paramRequired = TRUE;
 #ifdef	WIN32
 			else if (ci->server[0] == '\0')
 				paramRequired = TRUE;
@@ -209,8 +203,6 @@ inolog("DriverCompletion=%d\n", fDriverCompletion);
 	paramRequired = FALSE;
 	if (ci->database[0] == '\0')
 		paramRequired = TRUE;
-	else if (ci->port[0] == '\0')
-		paramRequired = TRUE;
 #ifdef	WIN32
 	else if (ci->server[0] == '\0')
 		paramRequired = TRUE;
@@ -219,13 +211,13 @@ inolog("DriverCompletion=%d\n", fDriverCompletion);
 	{
 		if (didUI)
 			return SQL_NO_DATA_FOUND;
-		CC_set_error(conn, CONN_OPENDB_ERROR, "connction string lacks some options", func);
+		CC_set_error(conn, CONN_OPENDB_ERROR, "connection string lacks some options", func);
 		return SQL_ERROR;
 	}
 
 inolog("before CC_connect\n");
 	/* do the actual connect */
-	retval = CC_connect(conn, password_required, salt);
+	retval = CC_connect(conn, salt);
 	if (retval < 0)
 	{							/* need a password */
 		if (fDriverCompletion == SQL_DRIVER_NOPROMPT)
@@ -237,10 +229,10 @@ inolog("before CC_connect\n");
 		else
 		{
 #ifdef WIN32
-			password_required = -retval;
 			goto dialog;
 #else
-			return SQL_ERROR;	/* until a better solution is found. */
+			/* Prompting for missing options is only supported on Windows. */
+			return SQL_ERROR;
 #endif
 		}
 	}
@@ -310,8 +302,6 @@ inolog("before CC_connect\n");
 	}
 #endif /* FORCE_PASSWORD_DISPLAY */
 
-	if (connStrIn)
-		free(connStrIn);
 	mylog("PGAPI_DriverConnect: returning %d\n", result);
 	return result;
 }
@@ -384,7 +374,7 @@ dconn_FDriverConnectProc(
 				SetFocus(GetDlgItem(hdlg, IDC_PORT));
 			else if (ci->username[0] == '\0')
 				SetFocus(GetDlgItem(hdlg, IDC_USER));
-			else if (ci->focus_password)
+			else if (ci->password_required)
 				SetFocus(GetDlgItem(hdlg, IDC_PASSWORD));
 			break;
 
@@ -435,7 +425,7 @@ dconn_get_attributes(copyfunc func, const char *connect_string, ConnInfo *ci)
 	char	*equals, *delp;
 	char	*strtok_arg;
 #ifdef	HAVE_STRTOK_R
-	char	*last;
+	char	*last = NULL;
 #endif /* HAVE_STRTOK_R */
 
 	if (our_connect_string = strdup(connect_string), NULL == our_connect_string)
@@ -525,9 +515,6 @@ dconn_get_attributes(copyfunc func, const char *connect_string, ConnInfo *ci)
 #endif /* FORCE_PASSWORD_DISPLAY */
 			mylog("attribute = '%s', value = '%s'\n", attribute, value);
 
-		if (!attribute || !value)
-			continue;
-
 		/* Copy the appropriate value to the conninfo  */
 		(*func)(ci, attribute, value);
 
@@ -539,7 +526,6 @@ dconn_get_attributes(copyfunc func, const char *connect_string, ConnInfo *ci)
 static void
 dconn_get_connect_attributes(const char *connect_string, ConnInfo *ci)
 {
-
 	CC_conninfo_init(ci, COPY_GLOBALS);
 	dconn_get_attributes(copyAttributes, connect_string, ci);
 }
