@@ -33,13 +33,15 @@
 #include "lobj.h"
 #include "pgapifunc.h"
 
+static const char *next_name_token(const char *s, size_t *len);
+
 /*extern GLOBAL_VALUES globals;*/
 
 
 /*		Perform a Prepare on the SQL statement */
 RETCODE		SQL_API
 PGAPI_Prepare(HSTMT hstmt,
-			  const SQLCHAR FAR * szSqlStr,
+			  const SQLCHAR * szSqlStr,
 			  SQLINTEGER cbSqlStr)
 {
 	CSTR func = "PGAPI_Prepare";
@@ -62,8 +64,8 @@ PGAPI_Prepare(HSTMT hstmt,
 	SC_set_prepared(self, NOT_YET_PREPARED);
 	switch (self->status)
 	{
-		case STMT_PREMATURE:
-			mylog("**** PGAPI_Prepare: STMT_PREMATURE, recycle\n");
+		case STMT_DESCRIBED:
+			mylog("**** PGAPI_Prepare: STMT_DESCRIBED, recycle\n");
 			SC_recycle_statement(self); /* recycle the statement, but do
 										 * not remove parameter bindings */
 			break;
@@ -141,7 +143,7 @@ inolog("SQLPrepare return=%d\n", retval);
 /*		Performs the equivalent of SQLPrepare, followed by SQLExecute. */
 RETCODE		SQL_API
 PGAPI_ExecDirect(HSTMT hstmt,
-				 const SQLCHAR FAR * szSqlStr,
+				 const SQLCHAR * szSqlStr,
 				 SQLINTEGER cbSqlStr,
 				 UWORD flag)
 {
@@ -174,10 +176,10 @@ inolog("a2\n");
 
 	/*
 	 * If an SQLPrepare was performed prior to this, but was left in the
-	 * premature state because an error occurred prior to SQLExecute then
+	 * described state because an error occurred prior to SQLExecute then
 	 * set the statement to finished so it can be recycled.
 	 */
-	if (stmt->status == STMT_PREMATURE)
+	if (stmt->status == STMT_DESCRIBED)
 		stmt->status = STMT_FINISHED;
 
 	stmt->statement_type = statement_type(stmt->statement);
@@ -207,8 +209,7 @@ inquireHowToPrepare(const StatementClass *stmt)
 
 	conn = SC_get_conn(stmt);
 	ci = &(conn->connInfo);
-	if (!ci->use_server_side_prepare ||
-		PG_VERSION_LT(conn, 7.3))
+	if (!ci->use_server_side_prepare)
 	{
 		/* Do prepare operations by the driver itself */
 		return PREPARE_BY_THE_DRIVER;
@@ -224,9 +225,15 @@ inquireHowToPrepare(const StatementClass *stmt)
 		}
 		if (stmt->multi_statement < 0)
 			PGAPI_NumParams((StatementClass *) stmt, &num_params);
-		if (stmt->multi_statement > 0) /* would divide the query into multiple commands and apply V3 parse requests for each of them */
-			ret = PROTOCOL_74(ci) ? PARSE_REQ_FOR_INFO : PREPARE_BY_THE_DRIVER;
-		else if (PROTOCOL_74(ci))
+		if (stmt->multi_statement > 0)
+		{
+			/*
+			 * divide the query into multiple commands and apply V3 parse
+			 * requests for each of them
+			 */
+			ret = PARSE_REQ_FOR_INFO;
+		}
+		else
 		{
 			if (SC_may_use_cursor(stmt))
 			{
@@ -240,17 +247,6 @@ inquireHowToPrepare(const StatementClass *stmt)
 			else
 				ret = PARSE_TO_EXEC_ONCE;
 		}
-		else
-		{
-			if (SC_may_use_cursor(stmt) &&
-			    (SQL_CURSOR_FORWARD_ONLY != stmt->options.cursor_type ||
-			    ci->drivers.use_declarefetch))
-				ret = PREPARE_BY_THE_DRIVER;
-			else if (SC_is_prepare_statement(stmt))
-				ret = USING_PREPARE_COMMAND;
-			else
-				ret = PREPARE_BY_THE_DRIVER;
-		}
 	}
 	if (SC_is_prepare_statement(stmt) && (PARSE_TO_EXEC_ONCE == ret))
 		ret = NAMED_PARSE_REQUEST;
@@ -265,8 +261,6 @@ decideHowToPrepare(StatementClass *stmt, BOOL force)
 
 	if (0 != method) /* a method was already determined */
 		return method;
-	if (stmt->inaccurate_result)
-		return method;
 	switch (stmt->prepare)
 	{
 		case NON_PREPARE_STATEMENT: /* not a prepare statement */
@@ -314,8 +308,6 @@ int HowToPrepareBeforeExec(StatementClass *stmt, BOOL checkOnly)
 	{
 		switch (how_to_prepare)
 		{
-			case USING_PREPARE_COMMAND:
-				return checkOnly ? doNothing : usingCommand;
 			case NAMED_PARSE_REQUEST:
 				return shouldParse;
 			case PARSE_TO_EXEC_ONCE:
@@ -335,9 +327,6 @@ int HowToPrepareBeforeExec(StatementClass *stmt, BOOL checkOnly)
 				return doNothing;
 		}
 	}
-	if (PG_VERSION_LE(conn, 7.3) ||
-	    !PROTOCOL_74(ci))
-		return nCallParse;
 
 	if (num_params > 0)
 	{
@@ -437,11 +426,9 @@ RETCODE	Exec_with_parameters_resolved(StatementClass *stmt, BOOL *exec_end)
 	cursor_type = stmt->options.cursor_type;
 	scroll_concurrency = stmt->options.scroll_concurrency;
 	/* Prepare the statement if possible at backend side */
-	if (!stmt->inaccurate_result)
-	{
-		if (HowToPrepareBeforeExec(stmt, FALSE) >= allowParse)
-			prepare_before_exec = TRUE;
-	}
+	if (HowToPrepareBeforeExec(stmt, FALSE) >= allowParse)
+		prepare_before_exec = TRUE;
+
 inolog("prepare_before_exec=%d srv=%d\n", prepare_before_exec, conn->connInfo.use_server_side_prepare);
 	/* Create the statement with parameters substituted. */
 	retval = copy_statement_with_parameters(stmt, prepare_before_exec);
@@ -455,55 +442,6 @@ inolog("prepare_before_exec=%d srv=%d\n", prepare_before_exec, conn->connInfo.us
 
 	mylog("   stmt_with_params = '%s'\n", stmt->stmt_with_params);
 
-	/*
-	 *	Dummy exection to get the column info.
-	 */
-	if (stmt->inaccurate_result && SC_is_parse_tricky(stmt))
-	{
-		BOOL		in_trans = CC_is_in_trans(conn);
-		BOOL		issued_begin = FALSE;
-		QResultClass *curres;
-
-		stmt->exec_current_row = -1;
-		*exec_end = TRUE;
-		if (!SC_is_pre_executable(stmt))
-			RETURN(SQL_SUCCESS)
-		if (strnicmp(stmt->stmt_with_params, "BEGIN;", 6) == 0)
-		{
-			/* do nothing */
-		}
-		else if (!in_trans)
-		{
-			if (issued_begin = CC_begin(conn), !issued_begin)
-			{
-				SC_set_error(stmt, STMT_EXEC_ERROR,  "Handle prepare error", func);
-				RETURN(SQL_ERROR)
-			}
-		}
-		/* we are now in a transaction */
-		res = CC_send_query(conn, stmt->stmt_with_params, NULL, 0, SC_get_ancestor(stmt));
-		if (!QR_command_maybe_successful(res))
-		{
-#ifndef	_LEGACY_MODE_
-			if (PG_VERSION_LT(conn, 8.0))
-				CC_abort(conn);
-#endif /* LEGACY_MODE_ */
-			SC_set_error(stmt, STMT_EXEC_ERROR, "Handle prepare error", func);
-			QR_Destructor(res);
-			RETURN(SQL_ERROR)
-		}
-		SC_set_Result(stmt, res);
-		for (curres = res; !curres->num_fields; curres = curres->next)
-			;
-		SC_set_Curres(stmt, curres);
-		if (CC_does_autocommit(conn))
-		{
-			if (issued_begin)
-				CC_commit(conn);
-		}
-		stmt->status = STMT_FINISHED;
-		RETURN(SQL_SUCCESS)
-	}
 	/*
 	 *	The real execution.
 	 */
@@ -532,30 +470,6 @@ mylog("about to begin SC_execute\n");
 			res = kres;
 		}
 	}
-#ifdef	NOT_USED
-	else if (SC_is_concat_prepare_exec(stmt))
-	{
-		if (res && QR_command_maybe_successful(res))
-		{
-			QResultClass	*kres;
-
-			kres = res->next;
-inolog("res->next=%p\n", kres);
-			res->next = NULL;
-			SC_set_Result(stmt, kres);
-			res = kres;
-			SC_set_prepared(stmt, PREPARED_PERMANENTLY);
-		}
-		else
-		{
-			retval = SQL_ERROR;
-			if (stmt->execute_statement)
-				free(stmt->execute_statement);
-			stmt->execute_statement = NULL;
-		}
-	}
-#endif /* NOT_USED */
-#if (ODBCVER >= 0x0300)
 	ipdopts = SC_get_IPDF(stmt);
 	if (ipdopts->param_status_ptr)
 	{
@@ -572,14 +486,12 @@ inolog("res->next=%p\n", kres);
 				break;
 		}
 	}
-#endif /* ODBCVER */
 	if (end_row = stmt->exec_end_row, end_row < 0)
 	{
 		apdopts = SC_get_APDF(stmt);
 		end_row = (SQLINTEGER) apdopts->paramset_size - 1;
 	}
-	if (stmt->inaccurate_result ||
-	    stmt->exec_current_row >= end_row)
+	if (stmt->exec_current_row >= end_row)
 	{
 		*exec_end = TRUE;
 		stmt->exec_current_row = -1;
@@ -588,7 +500,6 @@ inolog("res->next=%p\n", kres);
 		stmt->exec_current_row++;
 	if (res)
 	{
-#if (ODBCVER >= 0x0300)
 		EnvironmentClass *env = (EnvironmentClass *) CC_get_env(conn);
 		const char *cmd = QR_get_command(res);
 		SQLLEN	start_row;
@@ -612,7 +523,6 @@ inolog("res->next=%p\n", kres);
 			if (0 == count)
 				retval = SQL_NO_DATA;
 		}
-#endif /* ODBCVER */
 		stmt->diag_row_count = res->recent_processed_row_count;
 	}
 	/*
@@ -808,10 +718,10 @@ inolog("ret=%d\n", ret);
 		}
 	}
 cleanup:
-	if (SQL_NEED_DATA != ret)
-		SC_forget_unnamed(stmt); /* unnamed plan is no longer reliable */
+#ifdef NOT_USED
 	if (!SC_is_prepare_statement(stmt) && ONCE_DESCRIBED == stmt->prepared)
 		SC_set_prepared(stmt, NOT_YET_PREPARED);
+#endif
 	if (start_stmt || SQL_ERROR == ret)
 	{
 		if (stmt->lock_CC_for_rb > 0)
@@ -824,16 +734,22 @@ cleanup:
 	return ret;
 }
 
+/*
+ * Given a SQL statement, see if it is an INSERT INTO statement and extract
+ * the name of the table (with schema) of the table that was inserted to.
+ * (It is needed to resolve any @@identity references in the future.)
+ */
 void
 SC_setInsertedTable(StatementClass *stmt, RETCODE retval)
 {
-	const char *cmd = stmt->statement, *ptr;
+	const char *cmd = stmt->statement;
 	ConnectionClass	*conn;
 	size_t	len;
+	const char *token = NULL;
 
 	if (STMT_TYPE_INSERT != stmt->statement_type)
 		return;
-	if (SQL_NEED_DATA == retval)
+	if (!SQL_SUCCEEDED(retval))
 		return;
 	conn = SC_get_conn(stmt);
 #ifdef	NOT_USED /* give up the use of lastval() */
@@ -842,6 +758,15 @@ SC_setInsertedTable(StatementClass *stmt, RETCODE retval)
 #endif /* NOT_USED */
 	/*if (!CC_fake_mss(conn))
 		return;*/
+
+	/*
+	 * Parse a statement that was just executed. If it looks like an INSERT INTO
+	 * statement, try to extract the table name (and schema) of the table that
+	 * we inserted into.
+	 *
+	 * This is by no means fool-proof, we don't implement the whole backend
+	 * lexer and grammar here, but should handle most simple INSERT statements.
+	 */
 	while (isspace((UCHAR) *cmd)) cmd++;
 	if (!*cmd)
 		return;
@@ -861,41 +786,72 @@ SC_setInsertedTable(StatementClass *stmt, RETCODE retval)
 		return;
 	NULL_THE_NAME(conn->schemaIns);
 	NULL_THE_NAME(conn->tableIns);
-	if (!SQL_SUCCEEDED(retval))
-		return;
-	while (TRUE)
+
+	len = 0;
+	token = next_name_token(cmd, &len);
+	if (token && *token == IDENTIFIER_QUOTE)
+		STRN_TO_NAME(conn->tableIns, token + 1, len - 2);
+	else
+		STRN_TO_NAME(conn->tableIns, token, len);
+	token = next_name_token(token, &len);
+	if (token && *token == '.')
 	{
-		if (IDENTIFIER_QUOTE == *cmd)
-		{
-			if (ptr = strchr(cmd + 1, IDENTIFIER_QUOTE), NULL == ptr)	/* syntax error */
-			{
-				NULL_THE_NAME(conn->schemaIns);
-				NULL_THE_NAME(conn->tableIns);
-				break;
-			}
-			len = ptr - cmd - 1;
-			cmd++;
-			ptr++;
-		}
-		else
-		{
-			if (ptr = strchr(cmd + 1, '.'), NULL != ptr)
-				len = ptr - cmd;
+		token = next_name_token(token, &len);
+		if (token) {
+			if (NAME_IS_VALID(conn->tableIns))
+				MOVE_NAME(conn->schemaIns, conn->tableIns);
+			if (*token == IDENTIFIER_QUOTE)
+				STRN_TO_NAME(conn->tableIns, token + 1, len - 2);
 			else
-			{
-				ptr = cmd;
-				while (*ptr && !isspace((UCHAR) *ptr)) ptr++;
-				len = ptr - cmd;
-			}
+				STRN_TO_NAME(conn->tableIns, token, len);
 		}
-		if (NAME_IS_VALID(conn->tableIns))
-			MOVE_NAME(conn->schemaIns, conn->tableIns);
-		STRN_TO_NAME(conn->tableIns, cmd, len);
-		if ('.' == *ptr)
-			cmd = ptr + 1;
-		else
+	}
+
+	if (!NAME_IS_VALID(conn->tableIns))
+		NULL_THE_NAME(conn->schemaIns);
+}
+
+/*
+ * Returns the next token from a qualified or unqualified name.
+ *
+ * s is the previous token. On entry, *len is the length of the previous
+ * token; on return, it is the length of the next one. If the token is
+ * quoted, the quotes are included in the result. If no valid token is found,
+ * NULL is returned.
+ */
+static const char *
+next_name_token(const char *s, size_t *len)
+{
+	const char *p;
+
+	s += *len;
+	while (*s && isspace(*s)) ++s;
+
+	switch (*s) {
+		case '\0':
+			break;
+		case '.':
+			*len = 1;
+			return s;
+		case IDENTIFIER_QUOTE:
+			p = strchr(s + 1, IDENTIFIER_QUOTE);
+			if (p) {
+				*len = p - s + 1;
+				return s;
+			}
+			break;
+		default:
+			p = s;
+			while (*p && !isspace(*p) && *p != '.') ++p;
+			if (p) {
+				*len = p - s;
+				return s;
+			}
 			break;
 	}
+
+	*len = 0;
+	return NULL;
 }
 
 /*	Execute a prepared SQL statement */
@@ -916,34 +872,15 @@ PGAPI_Execute(HSTMT hstmt, UWORD flag)
 
 	conn = SC_get_conn(stmt);
 	apdopts = SC_get_APDF(stmt);
+
 	/*
-	 * If the statement is premature, it means we already executed it from
-	 * an SQLPrepare/SQLDescribeCol type of scenario.  So just return
-	 * success.
+	 * If the statement was previously described, just recycle the old result
+	 * set that contained just the column information.
 	 */
-	if (stmt->prepare && stmt->status == STMT_PREMATURE)
+	if (stmt->prepare && stmt->status == STMT_DESCRIBED)
 	{
-		if (stmt->inaccurate_result)
-		{
-			stmt->exec_current_row = -1;
-			SC_recycle_statement(stmt);
-		}
-		else
-		{
-			stmt->status = STMT_FINISHED;
-			if (NULL == SC_get_errormsg(stmt))
-			{
-				mylog("%s: premature statement but return SQL_SUCCESS\n", func);
-				retval = SQL_SUCCESS;
-				goto cleanup;
-			}
-			else
-			{
-				SC_set_error(stmt,STMT_INTERNAL_ERROR, "premature statement so return SQL_ERROR", func);
-				retval = SQL_ERROR;
-				goto cleanup;
-			}
-		}
+		stmt->exec_current_row = -1;
+		SC_recycle_statement(stmt);
 	}
 
 	mylog("%s: clear errors...\n", func);
@@ -1022,7 +959,7 @@ PGAPI_Execute(HSTMT hstmt, UWORD flag)
 			switch (nCallParse = HowToPrepareBeforeExec(stmt, TRUE))
 			{
 				case shouldParse:
-					if (retval = prepareParameters(stmt), SQL_ERROR == retval)
+					if (retval = prepareParameters(stmt, FALSE), SQL_ERROR == retval)
 						goto cleanup;
 					break;
 			}
@@ -1031,7 +968,6 @@ mylog("prepareParameters was %s called, prepare state:%d\n", shouldParse == nCal
 
 		if (ipdopts->param_processed_ptr)
 			*ipdopts->param_processed_ptr = 0;
-#if (ODBCVER >= 0x0300)
 		/*
 		 * Initialize param_status_ptr
 		 */
@@ -1040,7 +976,6 @@ mylog("prepareParameters was %s called, prepare state:%d\n", shouldParse == nCal
 			for (i = 0; i <= end_row; i++)
 				ipdopts->param_status_ptr[i] = SQL_PARAM_UNUSED;
 		}
-#endif /* ODBCVER */
 		if (recycle && !recycled)
 			SC_recycle_statement(stmt);
 		if (isSqlServr() &&
@@ -1052,7 +987,6 @@ mylog("prepareParameters was %s called, prepare state:%d\n", shouldParse == nCal
 	}
 
 next_param_row:
-#if (ODBCVER >= 0x0300)
 	if (apdopts->param_operation_ptr)
 	{
 		while (apdopts->param_operation_ptr[stmt->exec_current_row] == SQL_PARAM_IGNORE)
@@ -1071,12 +1005,11 @@ next_param_row:
 	 */
 	if (ipdopts->param_status_ptr)
 		ipdopts->param_status_ptr[stmt->exec_current_row] = SQL_PARAM_ERROR;
-#endif /* ODBCVER */
+
 	/*
 	 * Check if statement has any data-at-execute parameters when it is
 	 * not in SC_pre_execute.
 	 */
-	if (!stmt->pre_executing)
 	{
 		/*
 		 * The bound parameters could have possibly changed since the last
@@ -1111,7 +1044,7 @@ next_param_row:
 			/* Check for data at execution parameters */
 			if (apdopts->parameters[i].data_at_exec)
 			{
-				mylog("The %dth parameter of %d-row is data at exec(%d)\n", i, current_row, *pcVal);
+				mylog("The %dth parameter of %d-row is data at exec(%d)\n", i, current_row, pcVal ? (*pcVal) : -1);
 				if (stmt->data_at_exec < 0)
 					stmt->data_at_exec = 1;
 				else
@@ -1300,42 +1233,7 @@ PGAPI_Cancel(HSTMT hstmt)		/* Statement to cancel. */
 		 * The statement is not executing, and it's not waiting for params
 		 * either. Looks like it's not doing anything.
 		 */
-		if (conn->driver_version >= 0x0350)
-			return SQL_SUCCESS;
-		else
-		{
-			/*
-			 * MAJOR HACK for Windows to reset the driver manager's cursor
-			 * state: Because of what seems like a bug in the Odbc driver
-			 * manager, SQLCancel does not act like a SQLFreeStmt(CLOSE), as
-			 * many applications depend on this behavior.  So, this brute
-			 * force method calls the driver manager's function on behalf of
-			 * the application.
-			 */
-#ifdef WIN32
-			if (conn->connInfo.drivers.cancel_as_freestmt)
-			{
-	typedef SQLRETURN (SQL_API *SQLAPIPROC)();
-				HMODULE		hmodule;
-				SQLAPIPROC	addr;
-
-				hmodule = GetModuleHandle("ODBC32");
-				addr = (SQLAPIPROC) GetProcAddress(hmodule, "SQLFreeStmt");
-				ret = addr((char *) (stmt->phstmt) - 96, SQL_CLOSE);
-			}
-			else
-#endif /* WIN32 */
-			{
-				ENTER_STMT_CS(stmt);
-				SC_clear_error(hstmt);
-				ret = PGAPI_FreeStmt(hstmt, SQL_CLOSE);
-				if (stmt->internal)
-					ret = DiscardStatementSvp(stmt, ret, FALSE);
-				LEAVE_STMT_CS(stmt);
-			}
-			mylog("PGAPI_Cancel:  PGAPI_FreeStmt returned %d\n", ret);
-			return ret;
-		}
+		return SQL_SUCCESS;
 	}
 }
 
@@ -1347,11 +1245,11 @@ PGAPI_Cancel(HSTMT hstmt)		/* Statement to cancel. */
  */
 RETCODE		SQL_API
 PGAPI_NativeSql(HDBC hdbc,
-				const SQLCHAR FAR * szSqlStrIn,
+				const SQLCHAR * szSqlStrIn,
 				SQLINTEGER cbSqlStrIn,
-				SQLCHAR FAR * szSqlStr,
+				SQLCHAR * szSqlStr,
 				SQLINTEGER cbSqlStrMax,
-				SQLINTEGER FAR * pcbSqlStr)
+				SQLINTEGER * pcbSqlStr)
 {
 	CSTR func = "PGAPI_NativeSql";
 	size_t		len = 0;
@@ -1398,7 +1296,7 @@ PGAPI_NativeSql(HDBC hdbc,
  */
 RETCODE		SQL_API
 PGAPI_ParamData(HSTMT hstmt,
-				PTR FAR * prgbValue)
+				PTR * prgbValue)
 {
 	CSTR func = "PGAPI_ParamData";
 	StatementClass *stmt = (StatementClass *) hstmt, *estmt;
@@ -1555,7 +1453,7 @@ PGAPI_PutData(HSTMT hstmt,
 	ParameterInfoClass *current_param;
 	ParameterImplClass *current_iparam;
 	PutDataClass	*current_pdata;
-	char	   *buffer, *putbuf, *allocbuf = NULL;
+	char	   *putbuf, *allocbuf = NULL;
 	Int2		ctype;
 	SQLLEN		putlen;
 	BOOL		lenset = FALSE, handling_lo = FALSE;
@@ -1730,11 +1628,13 @@ PGAPI_PutData(HSTMT hstmt,
 		}
 		else
 		{
-			buffer = current_pdata->EXEC_buffer;
 			old_pos = *current_pdata->EXEC_used;
 			if (putlen > 0)
 			{
-				SQLLEN	used = *current_pdata->EXEC_used + putlen, allocsize;
+				SQLLEN	used = *current_pdata->EXEC_used + putlen;
+				SQLLEN allocsize;
+				char *buffer;
+
 				for (allocsize = (1 << 4); allocsize <= used; allocsize <<= 1) ;
 				mylog("        cbValue = %d, old_pos = %d, *used = %d\n", putlen, old_pos, used);
 
