@@ -224,7 +224,7 @@ schema_strcat(char *buf, const char *fmt, const SQLCHAR *s, SQLLEN len, const SQ
 		 * the CURRENT_SCHEMA() though it doesn't worth the
 		 * naming.
 		 */
-		if (conn->schema_support && tbname && (tbnmlen > 0 || tbnmlen == SQL_NTS))
+		if (tbname && (tbnmlen > 0 || tbnmlen == SQL_NTS))
 			return my_strcat(buf, fmt, CC_get_current_schema(conn), SQL_NTS);
 		return NULL;
 	}
@@ -252,13 +252,12 @@ my_trim(char *s)
  *	It can have 1 more parameter than my_strcat.
  */
 static char *
-my_strcat1(char *buf, const char *fmt, const char *s1, const char *s, ssize_t len)
+my_strcat1(char *buf, const char *fmt, const char *s1, const char *s)
 {
-	ssize_t	length = len;
-
-	if (s && (len > 0 || (len == SQL_NTS && (length = strlen(s)) > 0)))
+	if (s && s[0] != '\0')
 	{
 		size_t	pos = strlen(buf);
+		ssize_t	length = strlen(s);
 
 		if (s1)
 			sprintf(&buf[pos], fmt, s1, length, s);
@@ -270,15 +269,15 @@ my_strcat1(char *buf, const char *fmt, const char *s1, const char *s, ssize_t le
 }
 
 char *
-schema_strcat1(char *buf, const char *fmt, const char *s1, const char *s, ssize_t len, const SQLCHAR *tbname, int tbnmlen, ConnectionClass *conn)
+schema_strcat1(char *buf, const char *fmt, const char *s1, const char *s, const SQLCHAR *tbname, int tbnmlen, ConnectionClass *conn)
 {
-	if (!s || 0 == len)
+	if (!s || s[0] == '\0')
 	{
-		if (conn->schema_support && tbname && (tbnmlen > 0 || tbnmlen == SQL_NTS))
-			return my_strcat1(buf, fmt, s1, CC_get_current_schema(conn), SQL_NTS);
+		if (tbname && (tbnmlen > 0 || tbnmlen == SQL_NTS))
+			return my_strcat1(buf, fmt, s1, CC_get_current_schema(conn));
 		return NULL;
 	}
-	return my_strcat1(buf, fmt, s1, s, len);
+	return my_strcat1(buf, fmt, s1, s);
 }
 
 /*
@@ -317,6 +316,63 @@ snprintf_len(char *buf, size_t size, const char *format, ...)
 	return len;
 }
 
+/*
+ * Windows doesn't have snprintf(). It has _snprintf() which is similar,
+ * but it behaves differently wrt. truncation. This is a compatibility
+ * function that uses _snprintf() to provide POSIX snprintf() behavior.
+ *
+ * Our strategy, if the output doesn't fit, is to create a temporary buffer
+ * and call _snprintf() on that. If it still doesn't fit, enlarge the buffer
+ * and repeat.
+ */
+#ifdef WIN32
+static int
+posix_vsnprintf(char *str, size_t size, const char *format, va_list ap)
+{
+	int			len;
+	char	   *tmp;
+	size_t		newsize;
+
+	len = _vsnprintf(str, size, format, ap);
+	if (len < 0)
+	{
+		if (size == 0)
+			newsize = 100;
+		else
+			newsize = size;
+		do
+		{
+			newsize *= 2;
+			tmp = malloc(newsize);
+			if (!tmp)
+				return -1;
+			len = _vsnprintf(tmp, newsize, format, ap);
+			if (len >= 0)
+				memcpy(str, tmp, size);
+			free(tmp);
+		} while (len < 0);
+	}
+	if (len >= size && size > 0)
+	{
+		/* Ensure the buffer is NULL-terminated */
+		str[size - 1] = '\0';
+	}
+	return len;
+}
+
+int
+posix_snprintf(char *buf, size_t size, const char *format, ...)
+{
+	int		len;
+	va_list arglist;
+
+	va_start(arglist, format);
+	len = posix_vsnprintf(buf, size, format, arglist);
+	va_end(arglist);
+	return len;
+}
+#endif
+
 #ifndef	HAVE_STRLCAT
 size_t
 strlcat(char *dst, const char *src, size_t size)
