@@ -20,6 +20,12 @@
 #define	TRUE	1
 #endif
 
+typedef struct pg_CS
+{
+	char   *name;
+	int		code;
+} pg_CS;
+
 static pg_CS CS_Table[] =
 {
 	{ "SQL_ASCII",	SQL_ASCII },
@@ -76,8 +82,6 @@ static pg_CS CS_Alias[] =
 	{ "OTHER",	OTHER }
 };
 
-CSTR	OTHER_STRING = "OTHER";
-
 int
 pg_CS_code(const char *characterset_string)
 {
@@ -121,11 +125,13 @@ check_client_encoding(const pgNAME conn_settings)
 	for (cptr = SAFE_NAME(conn_settings); *cptr; cptr++)
 	{
 		if (in_quote)
+		{
 			if (LITERAL_QUOTE == *cptr)
 			{
 				in_quote = FALSE;
 				continue;
 			}
+		}
 		if (';' == *cptr)
 		{
 			allowed_cmd = TRUE;
@@ -404,99 +410,20 @@ pg_CS_stat(int stat,unsigned int character,int characterset_code)
 	return stat;
 }
 
-
-UCHAR *
-pg_mbschr(int csc, const UCHAR *string, unsigned int character)
-{
-	int			mb_st = 0;
-	const UCHAR *s, *rs = NULL;
-
-	for(s = string; *s ; s++)
-	{
-		mb_st = pg_CS_stat(mb_st, (UCHAR) *s, csc);
-		if (mb_st == 0 && (*s == character))
-		{
-			rs = s;
-			break;
-		}
-	}
-	return ((UCHAR *) rs);
-}
-
-size_t
-pg_mbslen(int csc, const UCHAR *string)
-{
-	UCHAR *s;
-	size_t	len;
-	int	cs_stat;
-	for (len = 0, cs_stat = 0, s = (UCHAR *) string; *s != 0; s++)
-	{
-		cs_stat = pg_CS_stat(cs_stat,(unsigned int) *s, csc);
-		if (cs_stat < 2)
-			len++;
-	}
-	return len;
-}
-
-static char *
-CC_lookup_cs_new(ConnectionClass *self)
-{
-	char		*encstr = NULL;
-	QResultClass	*res;
-
-	res = CC_send_query(self, "select pg_client_encoding()", NULL, IGNORE_ABORT_ON_CONN | ROLLBACK_ON_ERROR, NULL);
-	if (QR_command_maybe_successful(res))
-	{
-		const char *enc = QR_get_value_backend_text(res, 0, 0);
-
-		if (enc)
-			encstr = strdup(enc);
-	}
-	QR_Destructor(res);
-	return encstr;
-}
-static char *
-CC_lookup_cs_old(ConnectionClass *self)
-{
-	char		*encstr = NULL;
-	HSTMT		hstmt;
-	RETCODE		result;
-
-	result = PGAPI_AllocStmt(self, &hstmt, 0);
-	if (!SQL_SUCCEEDED(result))
-		return encstr;
-
-	result = PGAPI_ExecDirect(hstmt, (SQLCHAR *) "Show Client_Encoding", SQL_NTS, 0);
-	if (result == SQL_SUCCESS_WITH_INFO)
-	{
-		SQLCHAR		sqlState[8];
-		char		errormsg[128], enc[32];
-
-		if (PGAPI_Error(NULL, NULL, hstmt, sqlState, NULL, (SQLCHAR *) errormsg,
-			sizeof(errormsg), NULL) == SQL_SUCCESS &&
-		    sscanf(errormsg, "%*s %*s %*s %*s %*s %s", enc) > 0)
-			encstr = strdup(enc);
-	}
-	PGAPI_FreeStmt(hstmt, SQL_DROP);
-	return encstr;
-}
-
 /*
  *	This function works under Windows or Unicode case only.
  *	Simply returns NULL under other OSs.
  */
-const char * get_environment_encoding(const ConnectionClass *conn, const char *setenc, const char *currenc, BOOL bStartup)
+#ifndef	UNICODE_SUPPORT
+static const char *
+get_environment_encoding(const ConnectionClass *conn, const char *setenc, const char *currenc)
 {
 	const char *wenc = NULL;
 #ifdef	WIN32
 	int	acp;
 #endif /* WIN32 */
 
-#ifdef	UNICODE_SUPPORT
-	if (CC_is_in_unicode_driver(conn))
-		return "UTF8";
-#endif /* UNICODE_SUPPORT */
-	if (setenc && stricmp(setenc, OTHER_STRING))
+	if (setenc && stricmp(setenc, "OTHER"))
 		return setenc;
 	if (wenc = getenv("PGCLIENTENCODING"), NULL != wenc)
 		return wenc;
@@ -504,8 +431,7 @@ const char * get_environment_encoding(const ConnectionClass *conn, const char *s
 	acp = GetACP();
 	if (acp >= 1251 && acp <= 1258)
 	{
-		if (bStartup ||
-		    stricmp(currenc, "SQL_ASCII") == 0)
+		if (stricmp(currenc, "SQL_ASCII") == 0)
 			return wenc;
 	}
 	switch (acp)
@@ -514,12 +440,10 @@ const char * get_environment_encoding(const ConnectionClass *conn, const char *s
 			wenc = "SJIS";
 			break;
 		case 936:
-			if (!bStartup && PG_VERSION_GT(conn, 7.2))
-				wenc = "GBK";
+			wenc = "GBK";
 			break;
 		case 949:
-			if (!bStartup && PG_VERSION_GT(conn, 7.2))
-				wenc = "UHC";
+			wenc = "UHC";
 			break;
 		case 950:
 			wenc = "BIG5";
@@ -531,8 +455,7 @@ const char * get_environment_encoding(const ConnectionClass *conn, const char *s
 			wenc = "WIN1251";
 			break;
 		case 1256:
-			if (PG_VERSION_GE(conn, 7.3))
-				wenc = "WIN1256";
+			wenc = "WIN1256";
 			break;
 		case 1252:
 			if (strnicmp(currenc, "LATIN", 5) == 0)
@@ -566,22 +489,19 @@ const char * get_environment_encoding(const ConnectionClass *conn, const char *s
 #endif /* WIN32 */
 	return wenc;
 }
+#endif /* !UNICODE_SUPPORT */
 
 void
 CC_lookup_characterset(ConnectionClass *self)
 {
-	char	*encspec = NULL, *currenc = NULL, *tencstr;
+	const char *encspec, *currenc;
+	const char *tencstr;
 	CSTR func = "CC_lookup_characterset";
 
 	mylog("%s: entering...\n", func);
-	if (self->original_client_encoding)
-		encspec = strdup(self->original_client_encoding);
-	if (self->current_client_encoding)
-		currenc = strdup(self->current_client_encoding);
-	else if (PG_VERSION_LT(self, 7.2))
-		currenc = CC_lookup_cs_old(self);
-	else
-		currenc = CC_lookup_cs_new(self);
+	encspec = self->original_client_encoding;
+	currenc = PQparameterStatus(self->pqconn, "client_encoding");
+
 	tencstr = encspec ? encspec : currenc;
 	if (self->original_client_encoding)
 	{
@@ -597,7 +517,7 @@ CC_lookup_characterset(ConnectionClass *self)
 #ifndef	UNICODE_SUPPORT
 	else
 	{
-		const char *wenc = get_environment_encoding(self, encspec, currenc, FALSE);
+		const char *wenc = get_environment_encoding(self, encspec, currenc);
 		if (wenc && (!tencstr || stricmp(tencstr, wenc)))
 		{
 			QResultClass	*res;
@@ -614,10 +534,6 @@ CC_lookup_characterset(ConnectionClass *self)
 			{
 				self->original_client_encoding = strdup(wenc);
 				self->ccsc = pg_CS_code(self->original_client_encoding);
-				if (encspec)
-					free(encspec);
-				if (currenc)
-					free(currenc);
 				return;
 			}
 		}
@@ -625,9 +541,7 @@ CC_lookup_characterset(ConnectionClass *self)
 #endif /* UNICODE_SUPPORT */
 	if (tencstr)
 	{
-		self->original_client_encoding = tencstr;
-		if (encspec && currenc)
-			free(currenc);
+		self->original_client_encoding = strdup(tencstr);
 		self->ccsc = pg_CS_code(tencstr);
 		qlog("    [ Client encoding = '%s' (code = %d) ]\n", self->original_client_encoding, self->ccsc);
 		if (self->ccsc < 0)
