@@ -13,8 +13,6 @@
  */
 
 #include "psqlodbc.h"
-
-#if (ODBCVER >= 0x0300)
 #include <stdio.h>
 #include <string.h>
 
@@ -245,7 +243,7 @@ SQLGetDiagRecW(SQLSMALLINT fHandleType,
 			SQLULEN ulen = utf8_to_ucs2_lf(mtxt, tlen, FALSE, szErrorMsg, cbErrorMsgMax, TRUE);
 			if (ulen == (SQLULEN) -1)
 			{
-				tlen = (SQLSMALLINT) msgtowstr(NULL, mtxt,
+				tlen = (SQLSMALLINT) msgtowstr(mtxt,
 					(int) tlen, (LPWSTR) szErrorMsg, (int) cbErrorMsgMax);
 			}
 			else
@@ -380,7 +378,7 @@ SQLGetDiagFieldW(SQLSMALLINT	fHandleType,
 				SQLULEN ulen = (SQLSMALLINT) utf8_to_ucs2_lf(rgbD, blen, FALSE, (SQLWCHAR *) rgbDiagInfo, cbDiagInfoMax / WCLEN, TRUE);
 				if (ulen == (SQLULEN) -1)
 				{
-					blen = (SQLSMALLINT) msgtowstr(NULL, rgbD,
+					blen = (SQLSMALLINT) msgtowstr(rgbD,
 						(int) blen, (LPWSTR) rgbDiagInfo, (int) cbDiagInfoMax / WCLEN);
 				}
 				else
@@ -389,12 +387,6 @@ SQLGetDiagFieldW(SQLSMALLINT	fHandleType,
 					ret = SQL_SUCCESS_WITH_INFO;
 				if (pcbDiagInfo)
 				{
-#ifdef	WIN32
-					extern int	platformId;
-
-					if (VER_PLATFORM_WIN32_WINDOWS == platformId && NULL == rgbDiagInfo && 0 == cbDiagInfoMax)
-						blen++;
-#endif /* WIN32 */
 					*pcbDiagInfo = blen * WCLEN;
 				}
 			}
@@ -412,5 +404,3 @@ SQLGetDiagFieldW(SQLSMALLINT	fHandleType,
 
 	return ret;
 }
-
-#endif /* ODBCVER >= 0x0300 */
