@@ -51,10 +51,8 @@ SQLColumnsW(HSTMT StatementHandle,
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
 	StartRollbackState(stmt);
-#if (ODBCVER >= 0x0300)
 	if (stmt->options.metadata_id)
 		flag |= PODBC_NOT_SEARCH_PATTERN;
-#endif
 	if (SC_opencheck(stmt, func))
 		ret = SQL_ERROR;
 	else
@@ -119,7 +117,7 @@ SQLDriverConnectW(HDBC hdbc,
 				  SQLSMALLINT cbConnStrIn,
 				  SQLWCHAR *szConnStrOut,
 				  SQLSMALLINT cbConnStrOutMax,
-				  SQLSMALLINT FAR *pcbConnStrOut,
+				  SQLSMALLINT *pcbConnStrOut,
 				  SQLUSMALLINT fDriverCompletion)
 {
 	CSTR func = "SQLDriverConnectW";
@@ -303,8 +301,7 @@ SQLExecDirectW(HSTMT StatementHandle,
 	stxt = ucs2_to_utf8(StatementText, TextLength, &slen, FALSE);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	if (PG_VERSION_GE(SC_get_conn(stmt), 7.4))
-		flag |= PODBC_WITH_HOLD;
+	flag |= PODBC_WITH_HOLD;
 	StartRollbackState(stmt);
 	if (SC_opencheck(stmt, func))
 		ret = SQL_ERROR;
@@ -377,27 +374,10 @@ SQLGetInfoW(HDBC ConnectionHandle,
 	ENTER_CONN_CS(conn);
 	CC_set_in_unicode_driver(conn);
 	CC_clear_error(conn);
-#if (ODBCVER >= 0x0300)
-	mylog("[%s(30)]", func);
+	mylog("[%s]", func);
 	if ((ret = PGAPI_GetInfo(ConnectionHandle, InfoType, InfoValue,
 							 BufferLength, StringLength)) == SQL_ERROR)
-	{
-		if (conn->driver_version >= 0x0300)
-		{
-			CC_clear_error(conn);
-			ret = PGAPI_GetInfo30(ConnectionHandle, InfoType, InfoValue,
-								  BufferLength, StringLength);
-		}
-	}
-	if (SQL_ERROR == ret)
-		CC_log_error("SQLGetInfoW(30)", "", conn);
-#else
-	mylog("[%s]", func);
-	ret = PGAPI_GetInfo(ConnectionHandle, InfoType, InfoValue,
-						BufferLength, StringLength);
-	if (SQL_ERROR == ret)
 		CC_log_error("SQLGetInfoW", "", conn);
-#endif
 	LEAVE_CONN_CS(conn);
 	return ret;
 }
@@ -564,10 +544,8 @@ SQLTablesW(HSTMT StatementHandle,
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
 	StartRollbackState(stmt);
-#if (ODBCVER >= 0x0300)
 	if (stmt->options.metadata_id)
 		flag |= PODBC_NOT_SEARCH_PATTERN;
-#endif
 	if (SC_opencheck(stmt, func))
 		ret = SQL_ERROR;
 	else
@@ -619,10 +597,8 @@ SQLColumnPrivilegesW(HSTMT			hstmt,
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
 	StartRollbackState(stmt);
-#if (ODBCVER >= 0x0300)
 	if (stmt->options.metadata_id)
 		flag |= PODBC_NOT_SEARCH_PATTERN;
-#endif
 	if (SC_opencheck(stmt, func))
 		ret = SQL_ERROR;
 	else
@@ -835,10 +811,8 @@ SQLProcedureColumnsW(HSTMT			hstmt,
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
 	StartRollbackState(stmt);
-#if (ODBCVER >= 0x0300)
 	if (stmt->options.metadata_id)
 		flag |= PODBC_NOT_SEARCH_PATTERN;
-#endif
 	if (SC_opencheck(stmt, func))
 		ret = SQL_ERROR;
 	else
@@ -888,10 +862,8 @@ SQLProceduresW(HSTMT		hstmt,
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
 	StartRollbackState(stmt);
-#if (ODBCVER >= 0x0300)
 	if (stmt->options.metadata_id)
 		flag |= PODBC_NOT_SEARCH_PATTERN;
-#endif
 	if (SC_opencheck(stmt, func))
 		ret = SQL_ERROR;
 	else
@@ -938,10 +910,8 @@ SQLTablePrivilegesW(HSTMT			hstmt,
 	ENTER_STMT_CS((StatementClass *) hstmt);
 	SC_clear_error(stmt);
 	StartRollbackState(stmt);
-#if (ODBCVER >= 0x0300)
 	if (stmt->options.metadata_id)
 		flag |= PODBC_NOT_SEARCH_PATTERN;
-#endif
 	if (SC_opencheck(stmt, func))
 		ret = SQL_ERROR;
 	else
