@@ -83,12 +83,13 @@ getNextToken(
 	char escape_ch,
 	char *s, char *token, int smax, char *delim, char *quote, char *dquote, char *numeric)
 {
-	int			i = 0;
-	int		out = 0, taglen;
+	size_t		i = 0;
+	size_t		out = 0;
+	size_t		taglen;
 	char		qc, in_quote, in_dollar_quote, in_escape;
 	const	char	*tag, *tagend;
 	encoded_str	encstr;
-	char	literal_quote = LITERAL_QUOTE, identifier_quote = IDENTIFIER_QUOTE, dollar_quote = DOLLAR_QUOTE, escape_in_literal;
+	char		escape_in_literal;
 
 	if (smax <= 1)
 		return NULL;
@@ -135,19 +136,19 @@ getNextToken(
 		if (out == 0)
 		{
 			qc = s[i];
-			if (qc == dollar_quote)
+			if (qc == DOLLAR_QUOTE)
 			{
 				in_quote = in_dollar_quote = TRUE;
 				tag = s + i;
 				taglen = 1;
-				if (tagend = strchr(s + i + 1, dollar_quote), NULL != tagend)
+				if (tagend = strchr(s + i + 1, DOLLAR_QUOTE), NULL != tagend)
 					taglen = tagend - s - i + 1;
 				i += (taglen - 1);
 				encoded_position_shift(&encstr, taglen - 1);
 				if (quote)
 					*quote = TRUE;
 			}
-			else if (qc == literal_quote)
+			else if (qc == LITERAL_QUOTE)
 			{
 				in_quote = TRUE;
 				if (quote)
@@ -159,7 +160,7 @@ getNextToken(
 						escape_in_literal = ESCAPE_IN_LITERAL;
 				}
 			}
-			else if (qc == identifier_quote)
+			else if (qc == IDENTIFIER_QUOTE)
 			{
 				in_quote = TRUE;
 				if (dquote)
@@ -183,8 +184,18 @@ getNextToken(
 				else if (s[i] == qc)
 				{
 					if (!in_dollar_quote)
-						break;
-					if (strncmp(s + i, tag, taglen) == 0)
+					{
+						/*
+						 * Peek at the next byte to see if this is a '' or
+						 * "", i.e a quote character that has been escaped
+						 * by doubling it.
+						 */
+						if (s[i + 1] == qc)
+							i++;
+						else
+							break;
+					}
+					else if (strncmp(s + i, tag, taglen) == 0)
 					{
 						i += (taglen - 1);
 						encoded_position_shift(&encstr, taglen - 1);
@@ -192,7 +203,7 @@ getNextToken(
 					}
 					token[out++] = s[i];
 				}
-				else if (literal_quote == qc && s[i] == escape_in_literal)
+				else if (LITERAL_QUOTE == qc && s[i] == escape_in_literal)
 				{
 					in_escape = TRUE;
 				}
@@ -399,8 +410,7 @@ static BOOL CheckHasOids(StatementClass * stmt)
 				TI_set_hasoids(ti);
 				foundKey = TRUE;
 				STR_TO_NAME(ti->bestitem, OID_NAME);
-				sprintf(query, "\"%s\" = %%u", OID_NAME);
-				STRX_TO_NAME(ti->bestqual, query);
+				STRX_TO_NAME(ti->bestqual, "\"" OID_NAME "\" = %u");
 			}
 			TI_set_hasoids_checked(ti);
 			ti->table_oid = (OID) strtoul(QR_get_value_backend_text(res, 0, 1), NULL, 10);
@@ -603,7 +613,8 @@ inolog(" multi=%d\n", multi_table);
 /*
  *	SQLColAttribute tries to set the FIELD_INFO (using protocol 3).
  */
-static BOOL ColAttSet(StatementClass *stmt, TABLE_INFO *rti)
+static BOOL
+ColAttSet(StatementClass *stmt, TABLE_INFO *rti)
 {
 	QResultClass	*res = SC_get_Curres(stmt);
 	IRDFields	*irdflds = SC_get_IRDF(stmt);
@@ -615,14 +626,12 @@ static BOOL ColAttSet(StatementClass *stmt, TABLE_INFO *rti)
 	BOOL		fi_reuse, updatable, call_xxxxx;
 
 mylog("ColAttSet in\n");
-	if (rti)
-	{
-		if (reloid = rti->table_oid, 0 == reloid)
-			return FALSE;
-		if (0 != (rti->flags & TI_COLATTRIBUTE))
-			return TRUE;
-		col_info = rti->col_info;
-	}
+
+	if (reloid = rti->table_oid, 0 == reloid)
+		return FALSE;
+	if (0 != (rti->flags & TI_COLATTRIBUTE))
+		return TRUE;
+	col_info = rti->col_info;
 	if (!QR_command_maybe_successful(res))
 		return FALSE;
 	if (num_fields = QR_NumPublicResultCols(res), num_fields <= 0)
@@ -635,7 +644,7 @@ mylog("ColAttSet in\n");
 		fi = irdflds->fi;
 	}
 	setNumFields(irdflds, num_fields);
-	updatable = rti ? TI_is_updatable(rti) : FALSE;
+	updatable = TI_is_updatable(rti);
 mylog("updatable=%d tab=%d fields=%d", updatable, stmt->ntab, num_fields);
 	if (updatable)
 	{
@@ -694,8 +703,7 @@ mylog("->%d\n", updatable);
 			wfi->flag |= FIELD_COL_ATTRIBUTE;
 		}
 	}
-	if (rti)
-		rti->flags |= TI_COLATTRIBUTE;
+	rti->flags |= TI_COLATTRIBUTE;
 	return TRUE;
 }
 
@@ -709,21 +717,21 @@ COL_INFO **coli)
 	*coli = NULL;
 	if (NAME_IS_NULL(table_name))
 		return TRUE;
-	if (conn->schema_support)
+	if (NAME_IS_NULL(*schema_name))
 	{
-		if (NAME_IS_NULL(*schema_name))
+		const char *curschema = CC_get_current_schema(conn);
+		/*
+		 * Though current_schema() doesn't have
+		 * much sense in PostgreSQL, we first
+		 * check the current_schema() when no
+		 * explicit schema name is specified.
+		 */
+		if (curschema)
 		{
-			const char *curschema = CC_get_current_schema(conn);
-			/*
-			 * Though current_schema() doesn't have
-			 * much sense in PostgreSQL, we first
-			 * check the current_schema() when no
-			 * explicit schema name is specified.
-			 */
 			for (colidx = 0; colidx < conn->ntables; colidx++)
 			{
 				if (!NAMEICMP(conn->col_info[colidx]->table_name, table_name) &&
-				    !stricmp(SAFE_NAME(conn->col_info[colidx]->schema_name), curschema))
+					!stricmp(SAFE_NAME(conn->col_info[colidx]->schema_name), curschema))
 				{
 					mylog("FOUND col_info table='%s' current schema='%s'\n", PRINT_NAME(table_name), curschema);
 					found = TRUE;
@@ -731,54 +739,42 @@ COL_INFO **coli)
 					break;
 				}
 			}
-			if (!found)
-			{
-				QResultClass	*res;
-				char		token[256];
-				BOOL		tblFound = FALSE;
-
-				/*
-				 * We also have to check as follows.
-				 */
-				snprintf(token, sizeof(token),
-						 "select nspname from pg_namespace n, pg_class c"
-						 " where c.relnamespace=n.oid and c.oid='\"%s\"'::regclass",
-						 SAFE_NAME(table_name));
-				res = CC_send_query(conn, token, NULL, ROLLBACK_ON_ERROR | IGNORE_ABORT_ON_CONN, NULL);
-				if (QR_command_maybe_successful(res))
-				{
-					if (QR_get_num_total_tuples(res) == 1)
-					{
-						tblFound = TRUE;
-						STR_TO_NAME(*schema_name, QR_get_value_backend_text(res, 0, 0));
-					}
-				}
-				QR_Destructor(res);
-				if (!tblFound)
-					return FALSE;
-			}
 		}
-		if (!found && NAME_IS_VALID(*schema_name))
+		if (!found)
 		{
-			for (colidx = 0; colidx < conn->ntables; colidx++)
+			QResultClass	*res;
+			char		token[256];
+			BOOL		tblFound = FALSE;
+
+			/*
+			 * We also have to check as follows.
+			 */
+			snprintf(token, sizeof(token),
+					 "select nspname from pg_namespace n, pg_class c"
+					 " where c.relnamespace=n.oid and c.oid='\"%s\"'::regclass",
+					 SAFE_NAME(table_name));
+			res = CC_send_query(conn, token, NULL, ROLLBACK_ON_ERROR | IGNORE_ABORT_ON_CONN, NULL);
+			if (QR_command_maybe_successful(res))
 			{
-				if (!NAMEICMP(conn->col_info[colidx]->table_name, table_name) &&
-				    !NAMEICMP(conn->col_info[colidx]->schema_name, *schema_name))
+				if (QR_get_num_total_tuples(res) == 1)
 				{
-					mylog("FOUND col_info table='%s' schema='%s'\n", PRINT_NAME(table_name), PRINT_NAME(*schema_name));
-					found = TRUE;
-					break;
+					tblFound = TRUE;
+					STR_TO_NAME(*schema_name, QR_get_value_backend_text(res, 0, 0));
 				}
 			}
+			QR_Destructor(res);
+			if (!tblFound)
+				return FALSE;
 		}
 	}
-	else
+	if (!found && NAME_IS_VALID(*schema_name))
 	{
 		for (colidx = 0; colidx < conn->ntables; colidx++)
 		{
-			if (!NAMEICMP(conn->col_info[colidx]->table_name, table_name))
+			if (!NAMEICMP(conn->col_info[colidx]->table_name, table_name) &&
+			    !NAMEICMP(conn->col_info[colidx]->schema_name, *schema_name))
 			{
-				mylog("FOUND col_info table='%s'\n", table_name);
+				mylog("FOUND col_info table='%s' schema='%s'\n", PRINT_NAME(table_name), PRINT_NAME(*schema_name));
 				found = TRUE;
 				break;
 			}
