@@ -17,7 +17,6 @@
 #include "psqlodbc.h"
 #include "misc.h"
 
-#if (ODBCVER >= 0x0300)
 #include <stdio.h>
 #include <string.h>
 
@@ -263,7 +262,7 @@ PGAPI_GetDiagField(SQLSMALLINT HandleType, SQLHANDLE Handle,
 						case SQL_SUCCESS_WITH_INFO:
 							ret = SQL_SUCCESS;
 							if (pcbErrm > 0 && stmt->pgerror)
-							
+
 								*((SQLINTEGER *) DiagInfoPtr) = (pcbErrm  - 1)/ stmt->pgerror->recsize + 1;
 							break;
 						default:
@@ -792,9 +791,7 @@ IRDSetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 		case SQL_DESC_NUM_PREC_RADIX: /* read-only */
 		case SQL_DESC_OCTET_LENGTH: /* read-only */
 		case SQL_DESC_PRECISION: /* read-only */
-#if (ODBCVER >= 0x0350)
 		case SQL_DESC_ROWVER: /* read-only */
-#endif /* ODBCVER */
 		case SQL_DESC_SCALE: /* read-only */
 		case SQL_DESC_SCHEMA_NAME: /* read-only */
 		case SQL_DESC_SEARCHABLE: /* read-only */
@@ -919,9 +916,7 @@ inolog("IPDSetField RecN=%d allocated=%d\n", RecNumber, ipdopts->allocated);
 		case SQL_DESC_NUM_PREC_RADIX:
 		case SQL_DESC_OCTET_LENGTH:
 		case SQL_DESC_PRECISION:
-#if (ODBCVER >= 0x0350)
 		case SQL_DESC_ROWVER: /* read-only */
-#endif /* ODBCVER */
 		case SQL_DESC_TYPE_NAME: /* read-only */
 		case SQL_DESC_UNSIGNED: /* read-only */
 		default:ret = SQL_ERROR;
@@ -1296,9 +1291,7 @@ IRDGetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 		case SQL_DESC_NUM_PREC_RADIX: /* read-only */
 		case SQL_DESC_OCTET_LENGTH: /* read-only */
 		case SQL_DESC_PRECISION: /* read-only */
-#if (ODBCVER >= 0x0350)
 		case SQL_DESC_ROWVER: /* read-only */
-#endif /* ODBCVER */
 		case SQL_DESC_SCALE: /* read-only */
 		case SQL_DESC_SEARCHABLE: /* read-only */
 		case SQL_DESC_TYPE: /* read-only */
@@ -1321,7 +1314,8 @@ IRDGetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 			rettype = SQL_NTS;
 			bCallColAtt = TRUE;
 			break;
-		default:ret = SQL_ERROR;
+		default:
+			ret = SQL_ERROR;
 			DC_set_error(desc, DESC_INVALID_DESCRIPTOR_IDENTIFIER,
 				"invalid descriptor identifier");
 	}
@@ -1480,9 +1474,7 @@ inolog("IPDGetField RecN=%d allocated=%d\n", RecNumber, ipdopts->allocated);
 		case SQL_DESC_NULLABLE: /* read-only */
 		case SQL_DESC_NUM_PREC_RADIX:
 		case SQL_DESC_OCTET_LENGTH:
-#if (ODBCVER >= 0x0350)
 		case SQL_DESC_ROWVER: /* read-only */
-#endif /* ODBCVER */
 		case SQL_DESC_TYPE_NAME: /* read-only */
 		case SQL_DESC_UNSIGNED: /* read-only */
 		default:ret = SQL_ERROR;
@@ -1631,7 +1623,6 @@ PGAPI_SetConnectAttr(HDBC ConnectionHandle,
 		case SQL_ATTR_METADATA_ID:
 			conn->stmtOptions.metadata_id = CAST_UPTR(SQLUINTEGER, Value);
 			break;
-#if (ODBCVER >= 0x0351)
 		case SQL_ATTR_ANSI_APP:
 			if (SQL_AA_FALSE != CAST_PTR(SQLINTEGER, Value))
 			{
@@ -1645,7 +1636,6 @@ PGAPI_SetConnectAttr(HDBC ConnectionHandle,
 			}
 			/*return SQL_ERROR;*/
 			return SQL_SUCCESS;
-#endif /* ODBCVER */
 		case SQL_ATTR_ENLIST_IN_DTC:
 #ifdef	WIN32
 #ifdef	_HANDLE_ENLIST_IN_DTC_
@@ -1830,6 +1820,7 @@ PGAPI_SetDescField(SQLHDESC DescriptorHandle,
 			{
 				case DESC_INVALID_DESCRIPTOR_IDENTIFIER:
 					DC_set_errormsg(desc, "can't SQLSetDescField for this descriptor identifier");
+					break;
 				case DESC_INVALID_COLUMN_NUMBER_ERROR:
 					DC_set_errormsg(desc, "can't SQLSetDescField for this column number");
 					break;
@@ -1987,8 +1978,9 @@ RETCODE	bulk_ope_callback(RETCODE retcode, void *para)
 	{
 		if (SQL_ADD != s->operation)
 		{
-			memcpy(&global_idx, CALC_BOOKMARK_ADDR(bookmark, offset, bind_size, s->idx), sizeof(UInt4));
-			global_idx = SC_resolve_bookmark(global_idx);
+			UInt4 tmp;
+			memcpy(&tmp, CALC_BOOKMARK_ADDR(bookmark, offset, bind_size, s->idx), sizeof(UInt4));
+			global_idx = SC_resolve_bookmark(tmp);
 		}
 		/* Note opts->row_operation_ptr is ignored */
 		switch (s->operation)
@@ -2067,4 +2059,3 @@ PGAPI_BulkOperations(HSTMT hstmt, SQLSMALLINT operationX)
 		ret = DiscardStatementSvp(s.stmt, ret, FALSE);
 	return ret;
 }
-#endif /* ODBCVER >= 0x0300 */
