@@ -28,8 +28,8 @@ extern "C" {
 #define	PODBC_ERROR_CLEAR		(1L << 1)
 
 RETCODE SQL_API PGAPI_AllocConnect(HENV EnvironmentHandle,
-				   HDBC FAR * ConnectionHandle);
-RETCODE SQL_API PGAPI_AllocEnv(HENV FAR * EnvironmentHandle);
+				   HDBC * ConnectionHandle);
+RETCODE SQL_API PGAPI_AllocEnv(HENV * EnvironmentHandle);
 RETCODE SQL_API PGAPI_AllocStmt(HDBC ConnectionHandle,
 				HSTMT *StatementHandle, UDWORD flag);
 RETCODE SQL_API PGAPI_BindCol(HSTMT StatementHandle,
@@ -50,9 +50,9 @@ RETCODE SQL_API PGAPI_Connect(HDBC ConnectionHandle,
 		const SQLCHAR *UserName, SQLSMALLINT NameLength2,
 		const SQLCHAR *Authentication, SQLSMALLINT NameLength3);
 RETCODE SQL_API PGAPI_DriverConnect(HDBC hdbc, HWND hwnd,
-		const SQLCHAR FAR * szConnStrIn, SQLSMALLINT cbConnStrIn,
-		SQLCHAR FAR * szConnStrOut, SQLSMALLINT cbConnStrOutMax,
-		SQLSMALLINT FAR * pcbConnStrOut, SQLUSMALLINT fDriverCompletion);
+		const SQLCHAR * szConnStrIn, SQLSMALLINT cbConnStrIn,
+		SQLCHAR * szConnStrOut, SQLSMALLINT cbConnStrOutMax,
+		SQLSMALLINT * pcbConnStrOut, SQLUSMALLINT fDriverCompletion);
 RETCODE SQL_API PGAPI_BrowseConnect(HDBC hdbc,
 					const SQLCHAR *szConnStrIn, SQLSMALLINT cbConnStrIn,
 					SQLCHAR *szConnStrOut, SQLSMALLINT cbConnStrOutMax,
@@ -112,9 +112,6 @@ RETCODE SQL_API PGAPI_GetFunctions30(HDBC ConnectionHandle,
 RETCODE SQL_API PGAPI_GetInfo(HDBC ConnectionHandle,
 			  SQLUSMALLINT InfoType, PTR InfoValue,
 			  SQLSMALLINT BufferLength, SQLSMALLINT *StringLength);
-RETCODE SQL_API PGAPI_GetInfo30(HDBC ConnectionHandle,
-				SQLUSMALLINT InfoType, PTR InfoValue,
-				SQLSMALLINT BufferLength, SQLSMALLINT *StringLength);
 RETCODE SQL_API PGAPI_GetStmtOption(HSTMT StatementHandle,
 			SQLUSMALLINT Option, PTR Value,
 			SQLINTEGER *StringLength, SQLINTEGER BufferLength);
@@ -284,7 +281,6 @@ RETCODE SQL_API PGAPI_SetScrollOptions(
 					   SQLLEN crowKeyset,
 					   SQLUSMALLINT crowRowset);
 
-#if (ODBCVER >= 0x0300)
 RETCODE SQL_API PGAPI_GetDiagRec(SQLSMALLINT HandleType, SQLHANDLE Handle,
 		SQLSMALLINT RecNumber, SQLCHAR *Sqlstate,
 		SQLINTEGER *NativeError, SQLCHAR *MessageText,
@@ -300,13 +296,14 @@ RETCODE SQL_API PGAPI_GetStmtAttr(HSTMT StatementHandle,
 		SQLINTEGER Attribute, PTR Value,
 		SQLINTEGER BufferLength, SQLINTEGER *StringLength);
 
+/* Driver-specific connection attributes, for SQLSetConnectAttr() */
 enum {
 	SQL_ATTR_PGOPT_DEBUG = 65536
-	,SQL_ATTR_PGOPT_COMMLOG
-	,SQL_ATTR_PGOPT_PARSE
-	,SQL_ATTR_PGOPT_USE_DECLAREFETCH
-	,SQL_ATTR_PGOPT_SERVER_SIDE_PREPARE
-	,SQL_ATTR_PGOPT_FETCH
+	,SQL_ATTR_PGOPT_COMMLOG = 65537
+	,SQL_ATTR_PGOPT_PARSE = 65538
+	,SQL_ATTR_PGOPT_USE_DECLAREFETCH = 65539
+	,SQL_ATTR_PGOPT_SERVER_SIDE_PREPARE = 65540
+	,SQL_ATTR_PGOPT_FETCH = 65541
 };
 RETCODE SQL_API PGAPI_SetConnectAttr(HDBC ConnectionHandle,
 			SQLINTEGER Attribute, PTR Value,
@@ -331,7 +328,6 @@ RETCODE SQL_API PGAPI_DescError(SQLHDESC DescriptorHandle, SQLSMALLINT RecNumber
 			SQLCHAR *Sqlstate, SQLINTEGER *NativeError,
 			SQLCHAR *MessageText, SQLSMALLINT BufferLength,
 			SQLSMALLINT *TextLength, UWORD flag);
-#endif /* ODBCVER */
 
 #ifdef	__cplusplus
 }
