@@ -80,9 +80,7 @@ SQLSMALLINT	sqlTypes[] = {
 	/* SQL_BINARY, -- Commented out because VarBinary is more correct. */
 	SQL_BIT,
 	SQL_CHAR,
-#if (ODBCVER >= 0x0300)
 	SQL_TYPE_DATE,
-#endif /* ODBCVER */
 	SQL_DATE,
 	SQL_DECIMAL,
 	SQL_DOUBLE,
@@ -93,10 +91,8 @@ SQLSMALLINT	sqlTypes[] = {
 	SQL_NUMERIC,
 	SQL_REAL,
 	SQL_SMALLINT,
-#if (ODBCVER >= 0x0300)
 	SQL_TYPE_TIME,
 	SQL_TYPE_TIMESTAMP,
-#endif /* ODBCVER */
 	SQL_TIME,
 	SQL_TIMESTAMP,
 	SQL_TINYINT,
@@ -107,9 +103,7 @@ SQLSMALLINT	sqlTypes[] = {
 	SQL_WVARCHAR,
 	SQL_WLONGVARCHAR,
 #endif /* UNICODE_SUPPORT */
-#if (ODBCVER >= 0x0350)
 	SQL_GUID,
-#endif /* ODBCVER */
 /* AFAIK SQL_INTERVAL types cause troubles in some spplications */
 #ifdef	PG_INTERVAL_AS_SQL_INTERVAL
 	SQL_INTERVAL_MONTH,
@@ -129,7 +123,7 @@ SQLSMALLINT	sqlTypes[] = {
 	0
 };
 
-#if (ODBCVER >= 0x0300) && defined(ODBCINT64)
+#ifdef ODBCINT64
 #define	ALLOWED_C_BIGINT	SQL_C_SBIGINT
 /* #define	ALLOWED_C_BIGINT	SQL_C_CHAR */ /* Delphi should be either ? */
 #else
@@ -155,7 +149,6 @@ pg_true_type(const ConnectionClass *conn, OID type, OID basetype)
 #define MINUTE_BIT	(1 << 27)
 #define SECOND_BIT	(1 << 28)
 
-#if	(ODBCVER >= 0x0300)
 static SQLSMALLINT
 get_interval_type(Int4 atttypmod, const char **name)
 {
@@ -245,7 +238,6 @@ mylog("!!! %s atttypmod=%x\n", __FUNCTION__, atttypmod);
 		*name = "interval";
 	return 0;
 }
-#endif /* ODBCVER */
 
 static Int4
 getCharColumnSizeX(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longestlen, int handle_unknown_size_as)
@@ -285,12 +277,8 @@ getCharColumnSizeX(const ConnectionClass *conn, OID type, int atttypmod, int adt
 #endif /* UNICODE_SUPPORT */
 
 	if (maxsize == TEXT_FIELD_SIZE + 1) /* magic length for testing */
-	{
-		if (PG_VERSION_GE(conn, 7.1))
-			maxsize = 0;
-		else
-			maxsize = TEXT_FIELD_SIZE;
-	}
+		maxsize = 0;
+
 	/*
 	 * Static ColumnSize (i.e., the Maximum ColumnSize of the datatype) This
 	 * has nothing to do with a result set.
@@ -328,13 +316,7 @@ inolog("!!! adtsize_or_logngest=%d\n", adtsize_or_longestlen);
 		{
 			case PG_TYPE_VARCHAR:
 			case PG_TYPE_BPCHAR:
-#if (ODBCVER >= 0x0300)
 				return atttypmod;
-#else
-				if (CC_is_in_unicode_driver(conn) || conn->ms_jet)
-					return atttypmod;
-				return p;
-#endif /* ODBCVER */
 		}
 	}
 
@@ -420,14 +402,9 @@ static SQLSMALLINT
 getTimestampDecimalDigitsX(const ConnectionClass *conn, OID type, int atttypmod)
 {
 	mylog("%s: type=%d, atttypmod=%d\n", __FUNCTION__, type, atttypmod);
-
-	if (PG_VERSION_LT(conn, 7.2))
-		return 0;
-
 	return (atttypmod > -1 ? atttypmod : 6);
 }
 
-
 static SQLSMALLINT
 getTimestampColumnSizeX(const ConnectionClass *conn, OID type, int atttypmod)
 {
@@ -477,7 +454,6 @@ getIntervalColumnSize(OID type, int atttypmod)
 	mylog("%s: type=%d, atttypmod=%d\n", __FUNCTION__, type, atttypmod);
 
 	ttl = leading_precision;
-#if (ODBCVER >= 0x0300)
 	switch (get_interval_type(atttypmod, NULL))
 	{
 		case 0:
@@ -510,9 +486,6 @@ getIntervalColumnSize(OID type, int atttypmod)
 			ttl = 24;
 			break;
 	}
-#else
-	ttl += 9;
-#endif /* ODBCVER */
 	scale = getIntervalDecimalDigits(type, atttypmod);
 	return (scale > 0) ? ttl + 1 + scale : ttl;
 }
@@ -522,12 +495,10 @@ SQLSMALLINT
 pgtype_attr_to_concise_type(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longestlen)
 {
 	const ConnInfo	*ci = &(conn->connInfo);
-#if (ODBCVER >= 0x0300)
 	EnvironmentClass *env = (EnvironmentClass *) CC_get_env(conn);
 #ifdef	PG_INTERVAL_AS_SQL_INTERVAL
 	SQLSMALLINT	sqltype;
 #endif /* PG_INTERVAL_AS_SQL_INTERVAL */
-#endif /* ODBCVER */
 
 	switch (type)
 	{
@@ -590,11 +561,7 @@ pgtype_attr_to_concise_type(const ConnectionClass *conn, OID type, int atttypmod
 				return ci->int8_as;
 			if (conn->ms_jet)
 				return SQL_NUMERIC; /* maybe a little better than SQL_VARCHAR */
-#if (ODBCVER >= 0x0300)
 			return SQL_BIGINT;
-#else
-			return SQL_VARCHAR;
-#endif /* ODBCVER */
 
 		case PG_TYPE_NUMERIC:
 			return SQL_NUMERIC;
@@ -604,25 +571,19 @@ pgtype_attr_to_concise_type(const ConnectionClass *conn, OID type, int atttypmod
 		case PG_TYPE_FLOAT8:
 			return SQL_FLOAT;
 		case PG_TYPE_DATE:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_TYPE_DATE;
-#endif /* ODBCVER */
 			return SQL_DATE;
 		case PG_TYPE_TIME:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_TYPE_TIME;
-#endif /* ODBCVER */
 			return SQL_TIME;
 		case PG_TYPE_ABSTIME:
 		case PG_TYPE_DATETIME:
 		case PG_TYPE_TIMESTAMP_NO_TMZONE:
 		case PG_TYPE_TIMESTAMP:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_TYPE_TIMESTAMP;
-#endif /* ODBCVER */
 			return SQL_TIMESTAMP;
 		case PG_TYPE_MONEY:
 			return SQL_FLOAT;
@@ -635,9 +596,7 @@ pgtype_attr_to_concise_type(const ConnectionClass *conn, OID type, int atttypmod
 		case PG_TYPE_MACADDR:
 			return ALLOW_WCHAR(conn) ? SQL_WVARCHAR : SQL_VARCHAR;
 		case PG_TYPE_UUID:
-#if (ODBCVER >= 0x0350)
 			return SQL_GUID;
-#endif /* ODBCVER */
 			return ALLOW_WCHAR(conn) ? SQL_WVARCHAR : SQL_VARCHAR;
 
 		case PG_TYPE_INTERVAL:
@@ -677,12 +636,10 @@ pgtype_attr_to_sqldesctype(const ConnectionClass *conn, OID type, int atttypmod)
 #endif /* PG_INTERVAL_AS_SQL_INTERVAL */
 	switch (rettype = pgtype_attr_to_concise_type(conn, type, atttypmod, PG_UNSPECIFIED))
 	{
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_DATE:
 		case SQL_TYPE_TIME:
 		case SQL_TYPE_TIMESTAMP:
 			return SQL_DATETIME;
-#endif /* ODBCVER */
 	}
 	return rettype;
 }
@@ -694,7 +651,6 @@ pgtype_attr_to_datetime_sub(const ConnectionClass *conn, OID type, int atttypmod
 
 	switch (rettype = pgtype_attr_to_concise_type(conn, type, atttypmod, PG_UNSPECIFIED))
 	{
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_DATE:
 			return SQL_CODE_DATE;
 		case SQL_TYPE_TIME:
@@ -715,7 +671,6 @@ pgtype_attr_to_datetime_sub(const ConnectionClass *conn, OID type, int atttypmod
 		case SQL_INTERVAL_HOUR_TO_SECOND:
 		case SQL_INTERVAL_MINUTE_TO_SECOND:
 			return rettype - 100;
-#endif /* ODBCVER */
 	}
 	return -1;
 }
@@ -724,20 +679,16 @@ SQLSMALLINT
 pgtype_attr_to_ctype(const ConnectionClass *conn, OID type, int atttypmod)
 {
 	const ConnInfo	*ci = &(conn->connInfo);
-#if (ODBCVER >= 0x0300)
 	EnvironmentClass *env = (EnvironmentClass *) CC_get_env(conn);
 #ifdef	PG_INTERVAL_AS_SQL_INTERVAL
 	SQLSMALLINT	ctype;
 #endif /* PG_INTERVAL_A_SQL_INTERVAL */
-#endif /* ODBCVER */
 
 	switch (type)
 	{
 		case PG_TYPE_INT8:
-#if (ODBCVER >= 0x0300)
 			if (!conn->ms_jet)
 				return ALLOWED_C_BIGINT;
-#endif /* ODBCVER */
 			return SQL_C_CHAR;
 		case PG_TYPE_NUMERIC:
 			return SQL_C_CHAR;
@@ -753,25 +704,19 @@ pgtype_attr_to_ctype(const ConnectionClass *conn, OID type, int atttypmod)
 		case PG_TYPE_FLOAT8:
 			return SQL_C_DOUBLE;
 		case PG_TYPE_DATE:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_C_TYPE_DATE;
-#endif /* ODBCVER */
 			return SQL_C_DATE;
 		case PG_TYPE_TIME:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_C_TYPE_TIME;
-#endif /* ODBCVER */
 			return SQL_C_TIME;
 		case PG_TYPE_ABSTIME:
 		case PG_TYPE_DATETIME:
 		case PG_TYPE_TIMESTAMP_NO_TMZONE:
 		case PG_TYPE_TIMESTAMP:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_C_TYPE_TIMESTAMP;
-#endif /* ODBCVER */
 			return SQL_C_TIMESTAMP;
 		case PG_TYPE_MONEY:
 			return SQL_C_FLOAT;
@@ -789,10 +734,8 @@ pgtype_attr_to_ctype(const ConnectionClass *conn, OID type, int atttypmod)
 			return ALLOW_WCHAR(conn) ? SQL_C_WCHAR : SQL_C_CHAR;
 #endif /* UNICODE_SUPPORT */
 		case PG_TYPE_UUID:
-#if (ODBCVER >= 0x0350)
 			if (!conn->ms_jet)
 				return SQL_C_GUID;
-#endif /* ODBCVER */
 			return ALLOW_WCHAR(conn) ? SQL_C_WCHAR : SQL_C_CHAR;
 
 		case PG_TYPE_INTERVAL:
@@ -859,12 +802,7 @@ inolog("pgtype_to_name int4\n");
 		case PG_TYPE_ABSTIME:
 			return "abstime";
 		case PG_TYPE_DATETIME:
-			if (PG_VERSION_GT(conn, 7.1))
-				return "timestamptz";
-			else if (PG_VERSION_LT(conn, 7.0))
-				return "datetime";
-			else
-				return "timestamp";
+			return "timestamptz";
 		case PG_TYPE_TIMESTAMP_NO_TMZONE:
 			return "timestamp without time zone";
 		case PG_TYPE_TIMESTAMP:
@@ -885,12 +823,9 @@ inolog("pgtype_to_name int4\n");
 			return "cidr";
 		case PG_TYPE_UUID:
 			return "uuid";
-#if (ODBCVER >= 0x0300)
 		case PG_TYPE_INTERVAL:
 			get_interval_type(atttypmod, &tname);
 			return tname;
-#endif /* ODBCVER */
-
 		case PG_TYPE_LO_UNDEFINED:
 			return PG_TYPE_LO_NAME;
 
@@ -943,12 +878,7 @@ pgtype_attr_column_size(const ConnectionClass *conn, OID type, int atttypmod, in
 					value = NAME_FIELD_SIZE;
 #endif /* NAME_FIELD_SIZE */
 				if (0 == value)
-				{
-					if (PG_VERSION_GE(conn, 7.3))
-						value = NAMEDATALEN_V73;
-					else
-						value = NAMEDATALEN_V72;
-				}
+					value = NAMEDATALEN_V73;
 				return value;
 			}
 
@@ -1074,11 +1004,8 @@ pgtype_attr_display_size(const ConnectionClass *conn, OID type, int atttypmod, i
 		case PG_TYPE_INET:
 		case PG_TYPE_CIDR:
 			return sizeof("xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:255.255.255.255/128");
-#if (ODBCVER >= 0x0350)
 		case PG_TYPE_UUID:
 			return 36;
-#endif /* ODBCVER */
-
 		case PG_TYPE_INTERVAL:
 			return 30;
 
@@ -1135,10 +1062,7 @@ pgtype_attr_buffer_length(const ConnectionClass *conn, OID type, int atttypmod,
 		case PG_TYPE_CIDR:
 			return sizeof("xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:255.255.255.255/128");
 		case PG_TYPE_UUID:
-#if (ODBCVER >= 0x0350)
 			return 16;		/* sizeof(SQLGUID) */
-#endif /* ODBCVER */
-			return 36;
 
 			/* Character types use the default precision */
 		case PG_TYPE_VARCHAR:
@@ -1152,9 +1076,7 @@ pgtype_attr_buffer_length(const ConnectionClass *conn, OID type, int atttypmod,
 			if (CC_is_in_unicode_driver(conn))
 				return prec * WCLEN;
 #endif /* UNICODE_SUPPORT */
-			/* after 7.2 */
-			if (PG_VERSION_GE(conn, 7.2))
-				coef = conn->mb_maxbyte_per_char;
+			coef = conn->mb_maxbyte_per_char;
 			if (coef < 2 && (conn->connInfo).lf_conversion)
 				/* CR -> CR/LF */
 				coef = 2;
@@ -1276,11 +1198,13 @@ pgtype_attr_transfer_octet_length(const ConnectionClass *conn, OID type, int att
 {
 	int	coef = 1;
 	Int4	maxvarc, column_size;
+
 	switch (type)
 	{
 		case PG_TYPE_VARCHAR:
 		case PG_TYPE_BPCHAR:
 		case PG_TYPE_TEXT:
+		case PG_TYPE_UNKNOWN:
 			column_size = pgtype_attr_column_size(conn, type, atttypmod, PG_UNSPECIFIED, handle_unknown_size_as);
 			if (SQL_NO_TOTAL == column_size)
 				return column_size;
@@ -1288,9 +1212,7 @@ pgtype_attr_transfer_octet_length(const ConnectionClass *conn, OID type, int att
 			if (CC_is_in_unicode_driver(conn))
 				return column_size * WCLEN;
 #endif /* UNICODE_SUPPORT */
-			/* after 7.2 */
-			if (PG_VERSION_GE(conn, 7.2))
-				coef = conn->mb_maxbyte_per_char;
+			coef = conn->mb_maxbyte_per_char;
 			if (coef < 2 && (conn->connInfo).lf_conversion)
 				/* CR -> CR/LF */
 				coef = 2;
@@ -1310,6 +1232,135 @@ pgtype_attr_transfer_octet_length(const ConnectionClass *conn, OID type, int att
 }
 
 
+
+/*
+ * This is used when binding a query parameter, to decide which PostgreSQL
+ * datatype to send to the server, depending on the SQL datatype that was
+ * used in the SQLBindParameter call.
+ *
+ * For most types, this returns 0, which means that the server should treat
+ * the parameter the same as an untyped literal string, and deduce the type
+ * based on the context.
+ *
+ * This is different from sqltype_to_pgtype(), which doesn't return 0 but
+ * tries to give a closer match to the actual datatype.
+ */
+OID
+sqltype_to_bind_pgtype(const ConnectionClass *conn, SQLSMALLINT fSqlType)
+{
+	OID		pgType;
+	const ConnInfo	*ci = &(conn->connInfo);
+
+	switch (fSqlType)
+	{
+		case SQL_BINARY:
+		case SQL_VARBINARY:
+			pgType = PG_TYPE_BYTEA;
+			break;
+
+		case SQL_LONGVARBINARY:
+			/*
+			 * SQL_LONGVARBINARY could mean a large object, but it could also
+			 * mean bytea, in particular with ByteaAsLongVarBinary=1. With
+			 * large binary parameters, there isn't much danger of confusing
+			 * the datatype in practice - you typically just INSERT them
+			 * into a table or UPDATE them, and don't apply any operators to
+			 * them. So instead of trying to guess whether this is a 'bytea'
+			 * or large object, let the server decide.
+			 */
+			pgType = 0;
+			break;
+
+		case SQL_BIT:
+			pgType = ci->drivers.bools_as_char ? PG_TYPE_CHAR : PG_TYPE_BOOL;
+			break;
+
+		case SQL_TYPE_DATE:
+		case SQL_DATE:
+			pgType = PG_TYPE_DATE;
+			break;
+
+		case SQL_DOUBLE:
+		case SQL_FLOAT:
+			pgType = PG_TYPE_FLOAT8;
+			break;
+
+		case SQL_DECIMAL:
+		case SQL_NUMERIC:
+			pgType = PG_TYPE_NUMERIC;
+			break;
+
+		case SQL_BIGINT:
+			pgType = PG_TYPE_INT8;
+			break;
+
+		case SQL_INTEGER:
+			pgType = PG_TYPE_INT4;
+			break;
+
+		case SQL_REAL:
+			pgType = PG_TYPE_FLOAT4;
+			break;
+
+		case SQL_SMALLINT:
+		case SQL_TINYINT:
+			pgType = PG_TYPE_INT2;
+			break;
+
+		case SQL_TIME:
+		case SQL_TYPE_TIME:
+			pgType = PG_TYPE_TIME;
+			break;
+
+		case SQL_TIMESTAMP:
+		case SQL_TYPE_TIMESTAMP:
+			pgType = PG_TYPE_DATETIME;
+			break;
+
+		case SQL_GUID:
+			if (PG_VERSION_GE(conn, 8.3))
+				pgType = PG_TYPE_UUID;
+			else
+				pgType = 0;
+			break;
+
+		case SQL_INTERVAL_MONTH:
+		case SQL_INTERVAL_YEAR:
+		case SQL_INTERVAL_YEAR_TO_MONTH:
+		case SQL_INTERVAL_DAY:
+		case SQL_INTERVAL_HOUR:
+		case SQL_INTERVAL_MINUTE:
+		case SQL_INTERVAL_SECOND:
+		case SQL_INTERVAL_DAY_TO_HOUR:
+		case SQL_INTERVAL_DAY_TO_MINUTE:
+		case SQL_INTERVAL_DAY_TO_SECOND:
+		case SQL_INTERVAL_HOUR_TO_MINUTE:
+		case SQL_INTERVAL_HOUR_TO_SECOND:
+		case SQL_INTERVAL_MINUTE_TO_SECOND:
+			pgType = PG_TYPE_INTERVAL;
+			break;
+
+		case SQL_CHAR:
+#ifdef	UNICODE_SUPPORT
+		case SQL_WCHAR:
+#endif /* UNICODE_SUPPORT */
+		case SQL_LONGVARCHAR:
+#ifdef	UNICODE_SUPPORT
+		case SQL_WLONGVARCHAR:
+#endif /* UNICODE_SUPPORT */
+		case SQL_VARCHAR:
+#if	UNICODE_SUPPORT
+		case SQL_WVARCHAR:
+#endif /* UNICODE_SUPPORT */
+		default:
+			/* The default is to let the server choose */
+			pgType = 0;
+			break;
+	}
+
+	return pgType;
+}
+
 OID
 sqltype_to_pgtype(const ConnectionClass *conn, SQLSMALLINT fSqlType)
 {
@@ -1337,9 +1388,7 @@ sqltype_to_pgtype(const ConnectionClass *conn, SQLSMALLINT fSqlType)
 			pgType = ci->drivers.bools_as_char ? PG_TYPE_CHAR : PG_TYPE_BOOL;
 			break;
 
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_DATE:
-#endif /* ODBCVER */
 		case SQL_DATE:
 			pgType = PG_TYPE_DATE;
 			break;
@@ -1389,16 +1438,12 @@ sqltype_to_pgtype(const ConnectionClass *conn, SQLSMALLINT fSqlType)
 			break;
 
 		case SQL_TIME:
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_TIME:
-#endif /* ODBCVER */
 			pgType = PG_TYPE_TIME;
 			break;
 
 		case SQL_TIMESTAMP:
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_TIMESTAMP:
-#endif /* ODBCVER */
 			pgType = PG_TYPE_DATETIME;
 			break;
 
@@ -1416,12 +1461,10 @@ sqltype_to_pgtype(const ConnectionClass *conn, SQLSMALLINT fSqlType)
 			break;
 #endif /* UNICODE_SUPPORT */
 
-#if (ODBCVER >= 0x0350)
 		case SQL_GUID:
 			if (PG_VERSION_GE(conn, 8.3))
 				pgType = PG_TYPE_UUID;
 			break;
-#endif /* ODBCVER */
 
 		case SQL_INTERVAL_MONTH:
 		case SQL_INTERVAL_YEAR:
@@ -1468,19 +1511,25 @@ getAtttypmodEtc(const StatementClass *stmt, int col, int *adtsize_or_longestlen)
 					   atttypmod < 0 &&
 					   *adtsize_or_longestlen > 0)
 					{
-						int i, sval, maxscale = 0;
-						const char	*tval, *sptr;
+						SQLULEN		i;
+						size_t		sval, maxscale = 0;
+						const char *tval, *sptr;
 
 						for (i = 0; i < res->num_cached_rows; i++)
 						{
-							if (tval = QR_get_value_backend_text(res, i, col), NULL != tval)
+							tval = QR_get_value_backend_text(res, i, col);
+							if (NULL != tval)
 							{
-								if (sptr = strchr(tval, '.'), NULL != sptr)
-									if (sval = strlen(tval) - (sptr + 1 - tval), sval > maxscale)
+								sptr = strchr(tval, '.');
+								if (NULL != sptr)
+								{
+									sval = strlen(tval) - (sptr + 1 - tval);
+									if (sval > maxscale)
 										maxscale = sval;
+								}
 							}
 						}
-						*adtsize_or_longestlen += (maxscale << 16);
+						*adtsize_or_longestlen += (int) (maxscale << 16);
 					}
 				}
 			}
@@ -1666,12 +1715,8 @@ getCharColumnSize(const StatementClass *stmt, OID type, int col, int handle_unkn
 #endif /* UNICODE_SUPPORT */
 
 	if (maxsize == TEXT_FIELD_SIZE + 1) /* magic length for testing */
-	{
-		if (PG_VERSION_GE(conn, 7.1))
-			maxsize = 0;
-		else
-			maxsize = TEXT_FIELD_SIZE;
-	}
+		maxsize = 0;
+
 	/*
 	 * Static ColumnSize (i.e., the Maximum ColumnSize of the datatype) This
 	 * has nothing to do with a result set.
@@ -1705,13 +1750,7 @@ getCharColumnSize(const StatementClass *stmt, OID type, int col, int handle_unkn
 		{
 			case PG_TYPE_VARCHAR:
 			case PG_TYPE_BPCHAR:
-#if (ODBCVER >= 0x0300)
 				return attlen;
-#else
-				if (CC_is_in_unicode_driver(conn) || conn->ms_jet)
-					return attlen;
-				return p;
-#endif /* ODBCVER */
 		}
 	}
 
@@ -1757,8 +1796,6 @@ getTimestampDecimalDigits(const StatementClass *stmt, OID type, int col)
 
 	if (col < 0)
 		return 0;
-	if (PG_VERSION_LT(conn, 7.2))
-		return 0;
 
 	result = SC_get_Curres(stmt);
 
@@ -1886,9 +1923,7 @@ pgtype_transfer_octet_length(const StatementClass *stmt, OID type, int column_si
 			if (CC_is_in_unicode_driver(conn))
 				return column_size * WCLEN;
 #endif /* UNICODE_SUPPORT */
-			/* after 7.2 */
-			if (PG_VERSION_GE(conn, 7.2))
-				coef = conn->mb_maxbyte_per_char;
+			coef = conn->mb_maxbyte_per_char;
 			if (coef < 2 && (conn->connInfo).lf_conversion)
 				/* CR -> CR/LF */
 				coef = 2;
@@ -2203,14 +2238,9 @@ sqltype_to_default_ctype(const ConnectionClass *conn, SQLSMALLINT sqltype)
 		case SQL_LONGVARCHAR:
 		case SQL_DECIMAL:
 		case SQL_NUMERIC:
-#if (ODBCVER < 0x0300)
-		case SQL_BIGINT:
-			return SQL_C_CHAR;
-#else
 			return SQL_C_CHAR;
 		case SQL_BIGINT:
 			return ALLOWED_C_BIGINT;
-#endif
 
 #ifdef	UNICODE_SUPPORT
 		case SQL_WCHAR:
@@ -2254,7 +2284,6 @@ sqltype_to_default_ctype(const ConnectionClass *conn, SQLSMALLINT sqltype)
 		case SQL_TIMESTAMP:
 			return SQL_C_TIMESTAMP;
 
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_DATE:
 			return SQL_C_TYPE_DATE;
 
@@ -2263,15 +2292,12 @@ sqltype_to_default_ctype(const ConnectionClass *conn, SQLSMALLINT sqltype)
 
 		case SQL_TYPE_TIMESTAMP:
 			return SQL_C_TYPE_TIMESTAMP;
-#endif /* ODBCVER */
 
-#if (ODBCVER >= 0x0350)
 		case SQL_GUID:
 			if (conn->ms_jet)
 				return SQL_C_CHAR;
 			else
 				return SQL_C_GUID;
-#endif /* ODBCVER */
 
 		default:
 			/* should never happen */
@@ -2315,28 +2341,19 @@ ctype_length(SQLSMALLINT ctype)
 			return sizeof(UCHAR);
 
 		case SQL_C_DATE:
-#if (ODBCVER >= 0x0300)
 		case SQL_C_TYPE_DATE:
-#endif /* ODBCVER */
 			return sizeof(DATE_STRUCT);
 
 		case SQL_C_TIME:
-#if (ODBCVER >= 0x0300)
 		case SQL_C_TYPE_TIME:
-#endif /* ODBCVER */
 			return sizeof(TIME_STRUCT);
 
 		case SQL_C_TIMESTAMP:
-#if (ODBCVER >= 0x0300)
 		case SQL_C_TYPE_TIMESTAMP:
-#endif /* ODBCVER */
 			return sizeof(TIMESTAMP_STRUCT);
 
-#if (ODBCVER >= 0x0350)
 		case SQL_C_GUID:
 			return sizeof(SQLGUID);
-#endif /* ODBCVER */
-#if (ODBCVER >= 0x0300)
 		case SQL_C_INTERVAL_YEAR:
 		case SQL_C_INTERVAL_MONTH:
 		case SQL_C_INTERVAL_YEAR_TO_MONTH:
@@ -2356,7 +2373,6 @@ ctype_length(SQLSMALLINT ctype)
 		case SQL_C_SBIGINT:
 		case SQL_C_UBIGINT:
 			return sizeof(SQLBIGINT);
-#endif /* ODBCVER */
 
 		case SQL_C_BINARY:
 		case SQL_C_CHAR:
