@@ -71,20 +71,22 @@ void		debug_memory_check(void);
 #include <delayimp.h>
 #endif /* WIN32 */
 /* Must come before sql.h */
-#ifndef ODBCVER
-#define ODBCVER						0x0250
-#endif   /* ODBCVER_REP */
+#define ODBCVER					0x0351
 
-#define NAMEDATALEN_V72					32
+/*
+ * Default NAMEDATALEN value in the server. The server can be compiled with
+ * a different value, but this will do.
+ */
 #define NAMEDATALEN_V73					64
+
 #ifndef NAMESTORAGELEN
 #define NAMESTORAGELEN					64
-#endif   /* NAMEDATALEN */
-
+#endif   /* NAMESTORAGELEN */
 
 #ifndef	WIN32
 #undef	WIN_MULTITHREAD_SUPPORT
 #endif
+
 #if defined(WIN32) || defined(WITH_UNIXODBC) || defined(WITH_IODBC)
 #include <sql.h>
 #include <sqlext.h>
@@ -111,8 +113,6 @@ void		debug_memory_check(void);
 #include <odbcinst.h>
 #elif defined(WITH_IODBC)
 #include <iodbcinst.h>
-#else
-#include "gpps.h"
 #endif
 
 #ifdef	__cplusplus
@@ -125,9 +125,6 @@ extern "C" {
 #define UInt2 unsigned short
 typedef	UInt4	OID;
 
-#define	FORMAT_INT4	"%d"	/* Int4 */
-#define	FORMAT_UINT4	"%u"	/* UInt4 */
-
 #ifndef SQL_TRUE
 #define SQL_TRUE TRUE
 #endif /* SQL_TRUE */
@@ -224,7 +221,9 @@ typedef double SDOUBLE;
 #define	FALSE	(BOOL)0
 #endif /* FALSE */
 #else
-#define snprintf _snprintf
+#define snprintf posix_snprintf
+extern int posix_snprintf(char *buf, size_t size, const char *format, ...);
+
 #ifndef strdup
 #define strdup _strdup
 #endif /* strdup */
@@ -253,12 +252,8 @@ typedef double SDOUBLE;
 #define DBMS_NAME_UNICODE		"PostgreSQL Unicode"
 #define DBMS_NAME_ANSI			"PostgreSQL ANSI"
 
-#if (ODBCVER >= 0x0300)
-#if (ODBCVER >= 0x0351)
 #define DRIVER_ODBC_VER				"03.51"
-#else
-#define DRIVER_ODBC_VER				"03.00"
-#endif /* ODBCVER 0x0351 */
+
 #ifndef DBMS_NAME
 #ifdef	UNICODE_SUPPORT
 #define DBMS_NAME				DBMS_NAME_UNICODE
@@ -266,28 +261,17 @@ typedef double SDOUBLE;
 #define DBMS_NAME				DBMS_NAME_ANSI
 #endif /* UNICODE_SUPPORT */
 #endif /* DBMS_NAME */
-#else
-#define DRIVER_ODBC_VER				"02.50"
+
 #ifndef DBMS_NAME
 #define DBMS_NAME				"PostgreSQL Legacy"
 #endif   /* DBMS_NAME */
-#endif   /* ODBCVER */
-
 #ifdef WIN32
-#if (ODBCVER >= 0x0300)
 #ifdef	UNICODE_SUPPORT
-#if (ODBCVER >= 0x0350)
 #define DRIVER_FILE_NAME			"PSQLODBC35W.DLL"
 #else
-#define DRIVER_FILE_NAME			"PSQLODBC30W.DLL"
-#endif /* ODBCVER 0x0350 */
-#else
 #define DRIVER_FILE_NAME			"PSQLODBC.DLL"
 #endif   /* UNICODE_SUPPORT */
 #else
-#define DRIVER_FILE_NAME			"PSQLODBC25.DLL"
-#endif   /* ODBCVER 0x0300 */
-#else
 #ifdef  UNICODE_SUPPORT
 #define DRIVER_FILE_NAME                        "psqlodbcw.so"
 #else
@@ -312,24 +296,15 @@ BOOL isSqlServr(void);
 #define	PG_LINEFEED					'\n'
 
 /* Limits */
-#define BLCKSZ						4096
 #define MAXPGPATH					1024
 
-#define MAX_MESSAGE_LEN				65536		/* This puts a limit on
-												 * query size but I don't */
  /* see an easy way round this - DJP 24-1-2001 */
 #define MAX_CONNECT_STRING			4096
-#define ERROR_MSG_LENGTH			4096
 #define FETCH_MAX					100 /* default number of rows to cache
 										 * for declare/fetch */
 #define TUPLE_MALLOC_INC			100
-#define SOCK_BUFFER_SIZE			4096		/* default socket buffer
-												 * size */
 #define MAX_CONNECTIONS				128 /* conns per environment
 										 * (arbitrary)	*/
-#define MAX_FIELDS					512
-#define BYTELEN						8
-#define VARHDRSZ					sizeof(Int4)
 
 #ifdef	NAMEDATALEN
 #define MAX_SCHEMA_LEN				NAMEDATALEN
@@ -370,18 +345,9 @@ BOOL isSqlServr(void);
 #endif /* _MAX_PATH */
 #endif /* PATH_MAX */
 
-#define PG62						"6.2"		/* "Protocol" key setting
-												 * to force Postgres 6.2 */
-#define PG63						"6.3"		/* "Protocol" key setting
-												 * to force postgres 6.3 */
-#define PG64						"6.4"
-#define PG74REJECTED					"reject7.4"
-#define PG74						"7.4"
-
 typedef struct ConnectionClass_ ConnectionClass;
 typedef struct StatementClass_ StatementClass;
 typedef struct QResultClass_ QResultClass;
-typedef struct SocketClass_ SocketClass;
 typedef struct BindInfoClass_ BindInfoClass;
 typedef struct ParameterInfoClass_ ParameterInfoClass;
 typedef struct ParameterImplClass_ ParameterImplClass;
@@ -440,11 +406,6 @@ do { \
 	else \
 		(the_name).name = NULL; \
 } while (0)
-#define	NAME_TO_STR(str, the_name) \
-do {\
-	if ((the_name).name) strcpy(str, (the_name).name); \
-	else *str = '\0'; \
-} while (0)
 #define	NAME_TO_NAME(to, from) \
 do { \
 	if ((to).name) \
@@ -471,14 +432,11 @@ typedef struct GlobalValues_
 {
 	pgNAME		drivername;
 	int			fetch_max;
-	int			socket_buffersize;
 	int			unknown_sizes;
 	int			max_varchar_size;
 	int			max_longvarchar_size;
 	char		debug;
 	char		commlog;
-	char		disable_optimizer;
-	char		ksqo;
 	char		unique_index;
 	char		onlyread;		/* readonly is reserved on Digital C++
 								 * compiler */
@@ -488,7 +446,6 @@ typedef struct GlobalValues_
 	char		bools_as_char;
 	char		lie;
 	char		parse;
-	char		cancel_as_freestmt;
 	char		extra_systable_prefixes[MEDIUM_REGISTRY_LEN];
 	char		protocol[SMALL_REGISTRY_LEN];
 	pgNAME		conn_settings;
@@ -507,9 +464,7 @@ typedef struct StatementOptions_
 	SQLUINTEGER		retrieve_data;
 	SQLUINTEGER		use_bookmarks;
 	void			*bookmark_ptr;
-#if (ODBCVER >= 0x0300)
 	SQLUINTEGER		metadata_id;
-#endif /* ODBCVER */
 } StatementOptions;
 
 /*	Used to pass extra query info to send_query */
@@ -534,8 +489,8 @@ typedef struct
 PG_ErrorInfo	*ER_Constructor(SDWORD errornumber, const char *errormsg);
 PG_ErrorInfo	*ER_Dup(const PG_ErrorInfo *from);
 void ER_Destructor(PG_ErrorInfo *);
-RETCODE SQL_API ER_ReturnError(PG_ErrorInfo **, SQLSMALLINT, UCHAR FAR *,
-		SQLINTEGER FAR *, UCHAR FAR *, SQLSMALLINT, SQLSMALLINT FAR *, UWORD);
+RETCODE SQL_API ER_ReturnError(PG_ErrorInfo **, SQLSMALLINT, UCHAR *,
+		SQLINTEGER *, UCHAR *, SQLSMALLINT, SQLSMALLINT *, UWORD);
 
 void		logs_on_off(int cnopen, int, int);
 
@@ -582,8 +537,8 @@ const pthread_mutexattr_t *getMutexAttr(void);
 SQLULEN	ucs2strlen(const SQLWCHAR *ucs2str);
 char	*ucs2_to_utf8(const SQLWCHAR *ucs2str, SQLLEN ilen, SQLLEN *olen, BOOL tolower);
 SQLULEN	utf8_to_ucs2_lf(const char * utf8str, SQLLEN ilen, BOOL lfconv, SQLWCHAR *ucs2str, SQLULEN buflen, BOOL errcheck);
-int	msgtowstr(const char *, const char *, int, LPWSTR, int);
-int	wstrtomsg(const char *, const LPWSTR, int, char *, int);
+int	msgtowstr(const char *, int, LPWSTR, int);
+int	wstrtomsg(const LPWSTR, int, char *, int);
 #define	utf8_to_ucs2(utf8str, ilen, ucs2str, buflen) utf8_to_ucs2_lf(utf8str, ilen, FALSE, ucs2str, buflen, FALSE)
 #endif /* UNICODE_SUPPORT */
 
@@ -594,7 +549,7 @@ int	wstrtomsg(const char *, const LPWSTR, int, char *, int);
 
 CSTR	NULL_STRING = "";
 CSTR	PRINT_NULL = "(null)";
-CSTR	OID_NAME = "oid";
+#define OID_NAME "oid"
 
 #ifdef	__cplusplus
 }
