@@ -25,14 +25,14 @@ typedef enum
 	STMT_ALLOCATED,				/* The statement handle is allocated, but
 								 * not used so far */
 	STMT_READY,					/* the statement is waiting to be executed */
-	STMT_PREMATURE,				/* ODBC states that it is legal to call
+	STMT_DESCRIBED,				/* ODBC states that it is legal to call
 								 * e.g. SQLDescribeCol before a call to
 								 * SQLExecute, but after SQLPrepare. To
 								 * get all the necessary information in
-								 * such a case, we simply execute the
-								 * query _before_ the actual call to
-								 * SQLExecute, so that statement is
-								 * considered to be "premature". */
+								 * such a case, we parse the query _before_
+								 * the actual call to SQLExecute, and the
+								 * result set contains only column information,
+								 * but no actual data. */
 	STMT_FINISHED,				/* statement execution has finished */
 	STMT_EXECUTING				/* statement execution is still going on */
 } STMT_Status;
@@ -175,6 +175,22 @@ typedef	struct
 	void			*data;
 }	NeedDataCallback;
 
+
+/*
+ * ProcessedStmt represents a fragment of the original SQL query, after
+ * converting ? markers to $n style, processing ODBC escapes, and splitting
+ * a multi-statement into individual statements. Each individual statement
+ * is represented by one ProcessedStmt struct.
+ */
+struct ProcessedStmt
+{
+	struct ProcessedStmt *next;
+	char	   *query;
+	int			num_params;		/* number of parameter markers in this,
+								 * fragment or -1 if not known */
+};
+typedef struct ProcessedStmt ProcessedStmt;
+
 /********	Statement Handle	***********/
 struct StatementClass_
 {
@@ -182,7 +198,7 @@ struct StatementClass_
 								 * statement belongs to */
 	QResultClass *result;		/* result of the current statement */
 	QResultClass *curres;		/* the current result in the chain */
-	HSTMT FAR  *phstmt;
+	HSTMT	   *phstmt;
 	StatementOptions options;
 	StatementOptions options_orig;
 	/* attached descriptor handles */
@@ -218,6 +234,12 @@ struct StatementClass_
 
 	char	   *statement;		/* if non--null pointer to the SQL
 					 * statement that has been executed */
+	/*
+	 * processed_statements is the SQL after splitting multi-statement into
+	 * parts, and replacing ? markers with $n style markers, or injecting the
+	 * values in UseServerSidePrepare=0 mode.
+	 */
+	ProcessedStmt *processed_statements;
 
 	TABLE_INFO	**ti;
 	Int2		ntab;
@@ -253,21 +275,16 @@ struct StatementClass_
 
 	char		*stmt_with_params;	/* statement after parameter
 							 * substitution */
-	Int4		stmt_size_limit; /* PG restriction */
 	SQLLEN		exec_start_row;
 	SQLLEN		exec_end_row;
 	SQLLEN		exec_current_row;
 
-	po_ind_t	pre_executing;	/* This statement is prematurely executing */
-	po_ind_t	inaccurate_result;	/* Current status is PREMATURE but
-						 * result is inaccurate */
 	unsigned char	miscinfo;
 	po_ind_t	updatable;
 	SQLLEN		diag_row_count;
 	char		*load_statement; /* to (re)load updatable individual rows */
-	char		*execute_statement; /* to execute the prepared plans */
-	Int4		from_pos;
-	Int4		where_pos;
+	ssize_t		from_pos;
+	ssize_t		where_pos;
 	SQLLEN		last_fetch_count_include_ommitted;
 	time_t		stmt_time;
 	/* SQL_NEED_DATA Callback list */
@@ -326,9 +343,6 @@ void SC_set_Result(StatementClass *self, QResultClass *res);
 #define	SC_clear_parse_method(s)	((s)->parse_method = 0)
 #define	SC_is_parse_forced(s)	(0 != ((s)->parse_method & 1L))
 #define	SC_set_parse_forced(s)	((s)->parse_method |= 1L)
-#define	SC_is_parse_tricky(s)	(0 != ((s)->parse_method & 2L))
-#define	SC_set_parse_tricky(s)	((s)->parse_method |= 2L)
-#define	SC_no_parse_tricky(s)	((s)->parse_method &= ~2L)
 
 #define	SC_cursor_is_valid(s)	(NAME_IS_VALID(s->cursor_name))
 #define	SC_cursor_name(s)	(SAFE_NAME(s->cursor_name))
@@ -336,11 +350,7 @@ void SC_set_Result(StatementClass *self, QResultClass *res);
 void	SC_reset_delegate(RETCODE, StatementClass *);
 StatementClass *SC_get_ancestor(StatementClass *);
 
-#if (ODBCVER >= 0x0300)
 #define	SC_is_lower_case(a, b) (a->options.metadata_id || b->connInfo.lower_case_identifier)
-#else
-#define	SC_is_lower_case(a, b) (b->connInfo.lower_case_identifier)
-#endif /* ODBCVER */
 
 #define	SC_MALLOC_return_with_error(t, tp, s, a, m, r) \
 do { \
@@ -370,7 +380,6 @@ enum {
 	  NON_PREPARE_STATEMENT = 0
 	, PREPARE_STATEMENT = 1
 	, PREPARE_BY_THE_DRIVER = (1L << 1)
-	, USING_PREPARE_COMMAND = (2L << 1)
 	, NAMED_PARSE_REQUEST = (3L << 1)
 	, PARSE_TO_EXEC_ONCE = (4L << 1)
 	, PARSE_REQ_FOR_INFO = (5L << 1)
@@ -383,20 +392,16 @@ enum
 	,PREPARING_PERMANENTLY
 	,PREPARING_TEMPORARILY
 	,PREPARED_PERMANENTLY
-	,PREPARED_TEMPORARILY
-	,ONCE_DESCRIBED
+	,PREPARED_TEMPORARILY /* Is currently, or once was, prepared as unnamed
+						   * statement. You must check
+						   * connection->unnamed_prepared_stmt to see if it
+						   * still is */
 };
 
 /*	misc info */
-#define SC_set_pre_executable(a) (a->miscinfo |= 1L)
-#define SC_no_pre_executable(a) (a->miscinfo &= ~1L)
-#define SC_is_pre_executable(a) ((a->miscinfo & 1L) != 0)
 #define SC_set_fetchcursor(a)	(a->miscinfo |= (1L << 1))
 #define SC_no_fetchcursor(a)	(a->miscinfo &= ~(1L << 1))
 #define SC_is_fetchcursor(a)	((a->miscinfo & (1L << 1)) != 0)
-#define SC_set_concat_prepare_exec(a)	(a->miscinfo |= (1L << 2))
-#define SC_no_concat_prepare_exec(a)	(a->miscinfo &= ~(1L << 2))
-#define SC_is_concat_prepare_exec(a)	((a->miscinfo & (1L << 2)) != 0)
 #define SC_set_with_hold(a)	(a->miscinfo |= (1L << 3))
 #define SC_set_without_hold(a)	(a->miscinfo &= ~(1L << 3))
 #define SC_is_with_hold(a)	((a->miscinfo & (1L << 3)) != 0)
@@ -420,7 +425,6 @@ enum
 #define SC_started_rbpoint(a)	((a->rbonerr & (1L << 4)) != 0)
 #define SC_unref_CC_error(a)	((a->ref_CC_error) = FALSE)
 #define SC_ref_CC_error(a)	((a->ref_CC_error) = TRUE)
-void SC_forget_unnamed(StatementClass *self);
 #define SC_can_parse_statement(a) (STMT_TYPE_SELECT == (a)->statement_type)
 /*
  * DECLARE CURSOR + FETCH can only be used with SELECT-type queries. And
@@ -430,13 +434,6 @@ void SC_forget_unnamed(StatementClass *self);
 	(SC_get_APDF(a)->paramset_size <= 1 &&								\
 	 (STMT_TYPE_SELECT == (a)->statement_type || STMT_TYPE_WITH == (a)->statement_type) )
 #define SC_may_fetch_rows(a) (STMT_TYPE_SELECT == (a)->statement_type || STMT_TYPE_WITH == (a)->statement_type)
-#define SC_can_req_colinfo(a) (STMT_TYPE_SELECT == (a)->statement_type || \
-		 STMT_TYPE_WITH == (a)->statement_type || \
-		((a)->prepare && \
-		 STMT_TYPE_INSERT <= (a)->statement_type && \
-		 STMT_TYPE_DELETE >= (a)->statement_type && \
-		 SC_get_conn((a))->connInfo.use_server_side_prepare))
-#define SC_determine_statement_type(a) (STMT_TYPE_SELECT != (a)->statement_type ? (a)->statement_type : ((a)->statement_type = || statement_type((a)->statement)))
 
 
 /* For Multi-thread */
@@ -471,7 +468,7 @@ int		statement_type(const char *statement);
 char		parse_statement(StatementClass *stmt, BOOL);
 char		parse_sqlsvr(StatementClass *stmt);
 SQLRETURN	SC_set_SS_columnkey(StatementClass *stmt);
-Int4		SC_pre_execute(StatementClass *self);
+Int4		SC_describe(StatementClass *self);
 char		SC_unbind_cols(StatementClass *self);
 char		SC_recycle_statement(StatementClass *self);
 
@@ -500,7 +497,7 @@ RETCODE		SC_pos_add(StatementClass *self, SQLSETPOSIROW irow);
 int		SC_set_current_col(StatementClass *self, int col);
 void		SC_setInsertedTable(StatementClass *, RETCODE);
 void		SC_scanQueryAndCountParams(const char *, const ConnectionClass *,
-			Int4 *next_cmd, SQLSMALLINT *num_params,
+			ssize_t *next_cmd, SQLSMALLINT *num_params,
 			po_ind_t *multi, po_ind_t *proc_return);
 
 BOOL	SC_IsExecuting(const StatementClass *self);
@@ -515,14 +512,8 @@ int		StartRollbackState(StatementClass *self);
 RETCODE		SetStatementSvp(StatementClass *self);
 RETCODE		DiscardStatementSvp(StatementClass *self, RETCODE, BOOL errorOnly);
 
-BOOL		SendParseRequest(StatementClass *self, const char *name,
-			const char *query, Int4 qlen, Int2 num_params);
-BOOL		SyncParseRequest(ConnectionClass *conn);
-BOOL		SendDescribeRequest(StatementClass *self, const char *name, BOOL paramAlso);
-BOOL		SendBindRequest(StatementClass *self, const char *name);
-BOOL		BuildBindRequest(StatementClass *stmt, const char *name);
-BOOL		SendExecuteRequest(StatementClass *stmt, const char *portal, UInt4 count);
-QResultClass	*SendSyncAndReceive(StatementClass *stmt, QResultClass *res, const char *comment);
+QResultClass *ParseAndDescribeWithLibpq(StatementClass *stmt, const char *plan_name, const char *query_p, Int2 num_params, const char *comment, QResultClass *res);
+
 /*
  *	Macros to convert global index <-> relative index in resultset/rowset
  */
